@@ -1,6 +1,6 @@
-/*
-   GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2004, 2006, 2007, 2009 Free Software Foundation, Inc.
+/* GNU Mailutils -- a suite of utilities for electronic mail
+   Copyright (C) 2004, 2006, 2007, 2009, 2010, 2011 Free Software
+   Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,10 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA
-*/
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>.  */
 
 #include <iostream>
 #include <mailutils/cpp/mailutils.h>
@@ -32,11 +29,11 @@ read_and_print (Stream *in, Stream& out)
 {
   char buffer[128];
   
-  in->sequential_readline (buffer, sizeof (buffer));
+  in->readline (buffer, sizeof (buffer));
   while (in->get_read_count ())
     {
-      out.sequential_write (buffer, in->get_read_count ());
-      in->sequential_readline (buffer, sizeof (buffer));
+      out.write (buffer, in->get_read_count ());
+      in->readline (buffer, sizeof (buffer));
     }
 }
 
@@ -46,7 +43,7 @@ create_filter (bool read_stdin, char *cmdline, int flags)
   try {
     if (read_stdin)
       {
-	StdioStream *in = new StdioStream (stdin, 0);
+	StdioStream *in = new StdioStream (MU_STDIN_FD, 0);
 	in->open ();
 	FilterProgStream *stream = new FilterProgStream (cmdline, in);
 	stream->open ();
@@ -95,7 +92,7 @@ main (int argc, char *argv[])
   stream = create_filter (read_stdin, cmdline, flags);
 
   try {
-    StdioStream out (stdout, 0);
+    StdioStream out (MU_STDOUT_FD, 0);
     out.open ();
 
     read_and_print (stream, out);
