@@ -1,5 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001, 2007, 2008, 2009 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2001, 2007, 2008, 2009, 2010, 2011 Free Software
+   Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,9 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include "imap4d.h"
 
@@ -36,27 +35,45 @@ imap4d_delete (struct imap4d_command *command, imap4d_tokbuf_t tok)
   const char *msg = "Completed";
   const char *delim = "/";
   char *name;
-
+  mu_mailbox_t tmpbox;
+  
   if (imap4d_tokbuf_argc (tok) != 3)
-    return util_finish (command, RESP_BAD, "Invalid arguments");
+    return io_completion_response (command, RESP_BAD, "Invalid arguments");
   name = imap4d_tokbuf_getarg (tok, IMAP4_ARG_1);
   if (!name || *name == '\0')
-    return util_finish (command, RESP_BAD, "Too few arguments");
+    return io_completion_response (command, RESP_BAD, "Too few arguments");
 
   /* It is an error to attempt to delele "INBOX or a mailbox
      name that dos not exists.  */
   if (mu_c_strcasecmp (name, "INBOX") == 0)
-    return util_finish (command, RESP_NO, "Already exist");
+    return io_completion_response (command, RESP_NO, "Already exist");
 
  /* Allocates memory.  */
   name = namespace_getfullpath (name, delim, NULL);
   if (!name)
-    return util_finish (command, RESP_NO, "Cannot remove");
+    return io_completion_response (command, RESP_NO, "Cannot remove");
 
-  if (remove (name) != 0)
+  rc = mu_mailbox_create (&tmpbox, name);
+  if (rc == 0)
+    {
+      imap4d_enter_critical ();
+      rc = mu_mailbox_remove (tmpbox);
+      imap4d_leave_critical ();
+      mu_mailbox_destroy (&tmpbox);
+      if (rc)
+	mu_diag_funcall (MU_DIAG_ERROR, "mu_mailbox_remove", name, rc);
+    }
+  else
+    {
+      rc = remove (name);
+      if (rc)
+	mu_diag_funcall (MU_DIAG_ERROR, "remove", name, errno);
+    }
+
+  if (rc)
     {
       rc = RESP_NO;
       msg = "Cannot remove";
     }
-  return util_finish (command, rc, msg);
+  return io_completion_response (command, rc, "%s", msg);
 }
