@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
    Copyright (C) 1999, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
-   2009, 2010 Free Software Foundation, Inc.
+   2009, 2010, 2011 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,9 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 #ifndef _IMAP4D_H
 #define _IMAP4D_H 1
@@ -88,25 +86,26 @@
 #include <mailutils/mailbox.h>
 #include <mailutils/message.h>
 #include <mailutils/mime.h>
-#include <mailutils/mutil.h>
+#include <mailutils/util.h>
 #include <mailutils/mu_auth.h>
 #include <mailutils/nls.h>
 #include <mailutils/parse822.h>
 #include <mailutils/registrar.h>
 #include <mailutils/stream.h>
+#include <mailutils/stdstream.h>
 #include <mailutils/tls.h>
 #include <mailutils/url.h>
 #include <mailutils/daemon.h>
 #include <mailutils/pam.h>
 #include <mailutils/acl.h>
 #include <mailutils/server.h>
-#include <mailutils/argcv.h>
+#include <mailutils/wordsplit.h>
 #include <mailutils/alloc.h>
-#include <mailutils/vartab.h>
 #include <mailutils/cctype.h>
 #include <mailutils/cstr.h>
+#include <mailutils/io.h>
+#include <mailutils/prog.h>
 
-#include <mu_asprintf.h>
 #include <mu_umaxtostr.h>
 #include <muaux.h>
 
@@ -154,7 +153,8 @@ struct imap4d_command
 #define ERR_TLS               6
 #define ERR_MAILBOX_CORRUPTED 7
 #define ERR_TERMINATE         8
-  
+#define ERR_STREAM_CREATE     9
+
 /* Namespace numbers */
 #define NS_PRIVATE 0
 #define NS_OTHER   1
@@ -202,20 +202,36 @@ extern int imap4d_transcript;
 extern mu_list_t imap4d_id_list;
 extern int imap4d_argc;                 
 extern char **imap4d_argv;
-  
-#ifndef HAVE_STRTOK_R
-extern char *strtok_r (char *s, const char *delim, char **save_ptr);
-#endif
+extern jmp_buf child_jmp;
 
 /* Input functions */
+extern mu_stream_t iostream;
+extern int  io_untagged_response (int, const char *, ...) MU_PRINTFLIKE(2,3);
+extern int  io_sendf (const char *, ...) MU_PRINTFLIKE(1,2);
+extern int  io_send_bytes (const char *buf, size_t size);
+extern int  io_send_qstring (const char *);
+extern int  io_send_literal (const char *);
+extern int  io_copy_out (mu_stream_t str, size_t size);
+extern int  io_completion_response (struct imap4d_command *, int,
+                                    const char *, ...) MU_PRINTFLIKE(3,4);
+extern int io_stream_completion_response (mu_stream_t str,
+					  struct imap4d_command *command,
+					  int rc, 
+					  const char *format, ...)
+                                    MU_PRINTFLIKE(4,5);
+void io_getline (char **pbuf, size_t *psize, size_t *pnbytes);
+void io_setio (int, int, int);
+void io_flush (void);
+int io_wait_input (int);
+  
 imap4d_tokbuf_t imap4d_tokbuf_init (void);
 void imap4d_tokbuf_destroy (imap4d_tokbuf_t *tok);
 int imap4d_tokbuf_argc (imap4d_tokbuf_t tok);
 char *imap4d_tokbuf_getarg (imap4d_tokbuf_t tok, int n);
 void imap4d_readline (imap4d_tokbuf_t tok);
-struct imap4d_tokbuf *imap4d_tokbuf_from_string (char *str);
-int imap4d_getline (char **pbuf, size_t *psize, size_t *pnbytes);
+imap4d_tokbuf_t imap4d_tokbuf_from_string (char *str);
 
+  
 #define IMAP4_ARG_TAG     0
 #define IMAP4_ARG_COMMAND 1
 #define IMAP4_ARG_1       2
@@ -268,7 +284,6 @@ extern int  imap4d_expunge (struct imap4d_command *, imap4d_tokbuf_t);
 extern int  imap4d_fetch (struct imap4d_command *, imap4d_tokbuf_t);
 extern int  imap4d_fetch0 (imap4d_tokbuf_t tok, int isuid, char **err_text);
 extern int  imap4d_list (struct imap4d_command *, imap4d_tokbuf_t);
-extern int  imap4d_lsub (struct imap4d_command *, imap4d_tokbuf_t);
 extern int  imap4d_login (struct imap4d_command *, imap4d_tokbuf_t);
 extern int  imap4d_logout (struct imap4d_command *, imap4d_tokbuf_t);
 extern int  imap4d_noop (struct imap4d_command *, imap4d_tokbuf_t);
@@ -282,13 +297,18 @@ extern int  imap4d_select_status (void);
 #ifdef WITH_TLS
 extern int  imap4d_starttls (struct imap4d_command *, imap4d_tokbuf_t);
 extern void starttls_init (void);
+void tls_encryption_on (void);
 #endif /* WITH_TLS */
 extern int  imap4d_status (struct imap4d_command *, imap4d_tokbuf_t);
 extern int  imap4d_store (struct imap4d_command *, imap4d_tokbuf_t);
 extern int  imap4d_store0 (imap4d_tokbuf_t, int, char **);
+
+mu_property_t open_subscription (void);
 extern int  imap4d_subscribe (struct imap4d_command *, imap4d_tokbuf_t);
-extern int  imap4d_uid (struct imap4d_command *, imap4d_tokbuf_t);
 extern int  imap4d_unsubscribe (struct imap4d_command *, imap4d_tokbuf_t);
+extern int  imap4d_lsub (struct imap4d_command *, imap4d_tokbuf_t);
+
+extern int  imap4d_uid (struct imap4d_command *, imap4d_tokbuf_t);
 extern int  imap4d_namespace (struct imap4d_command *, imap4d_tokbuf_t);
 extern int  imap4d_version (struct imap4d_command *, imap4d_tokbuf_t);
 extern int  imap4d_idle (struct imap4d_command *, imap4d_tokbuf_t);
@@ -299,8 +319,14 @@ extern int imap4d_check_home_dir (const char *dir, uid_t uid, gid_t gid);
 /* Shared between fetch and store */  
 extern void fetch_flags0 (const char *prefix, mu_message_t msg, int isuid);
 
+/* Permissions for creating intermediate directories.
+   FIXME: These should better be configurable. */
+#define MKDIR_PERMISSIONS 0700
+int make_interdir (const char *name, int delim, int perms);
+
 /* Synchronisation on simultaneous access.  */
 extern int imap4d_sync (void);
+extern void imap4d_sync_invalidate (void);
 extern int imap4d_sync_flags (size_t);
 extern size_t uid_to_msgno (size_t);
 extern void imap4d_set_observer (mu_mailbox_t mbox);
@@ -310,6 +336,9 @@ extern RETSIGTYPE imap4d_master_signal (int);
 extern RETSIGTYPE imap4d_child_signal (int);
 extern int imap4d_bye (int);
 extern int imap4d_bye0 (int reason, struct imap4d_command *command);
+void imap4d_enter_critical (void);
+void imap4d_leave_critical (void);
+
 
 /* Namespace functions */
 extern mu_list_t namespace[NS_MAX];
@@ -322,21 +351,16 @@ extern char *namespace_checkfullpath (const char *name, const char *pattern,
 				       const char *delim, int *pns);
 int imap4d_session_setup (char *username);
 int imap4d_session_setup0 (void);
+void imap4d_child_signal_setup (RETSIGTYPE (*handler) (int signo));
   
 /* Capability functions */
 extern void imap4d_capability_add (const char *str);
 extern void imap4d_capability_remove (const char *str);
 extern void imap4d_capability_init (void);
-  
+
 /* Helper functions.  */
-extern int  util_out (int, const char *, ...) MU_PRINTFLIKE(2,3);
-extern int  util_send (const char *, ...) MU_PRINTFLIKE(1,2);
-extern int  util_send_bytes (const char *buf, size_t size);
-extern int  util_send_qstring (const char *);
-extern int  util_send_literal (const char *);
+
 extern int  util_start (char *);
-extern int  util_finish (struct imap4d_command *, int, const char *, ...) 
-                         MU_PRINTFLIKE(3,4);
 extern int  util_getstate (void);
 extern int  util_do_command (imap4d_tokbuf_t);
 extern char *util_tilde_expansion (const char *, const char *);
@@ -344,32 +368,32 @@ extern char *util_getfullpath (const char *, const char *);
 extern int  util_msgset (char *, size_t **, int *, int);
 extern struct imap4d_command *util_getcommand (char *, 
                                                struct imap4d_command []);
-extern int util_parse_internal_date (char *date, time_t *timep);
-extern int util_parse_822_date (const char *date, time_t *timep);
-extern int util_parse_ctime_date (const char *date, time_t *timep);
+
+enum datetime_parse_mode     /* how to parse date/time strings */
+  {
+    datetime_default,        /* default mode */
+    datetime_date_only,      /* return only date part, ignore time and TZ */
+    datetime_time_only       /* return only time and TZ, ignore date */ 
+  };
+
+extern int util_parse_internal_date (char *date, time_t *timep,
+				     enum datetime_parse_mode flag);
+extern int util_parse_822_date (const char *date, time_t *timep,
+				enum datetime_parse_mode flag);
+extern int util_parse_ctime_date (const char *date, time_t *timep,
+				  enum datetime_parse_mode flag);
 extern char *util_strcasestr (const char *haystack, const char *needle);
 extern char *util_localname (void);
 
 extern int util_wcard_match (const char *string, const char *pattern,
 			     const char *delim);
+int util_format_attribute_flags (mu_stream_t str, int flags);
 void util_print_flags (mu_attribute_t attr);
 int util_attribute_to_type (const char *item, int *type);
 int util_type_to_attribute (int type, char **attr_str);
 int util_attribute_matches_flag (mu_attribute_t attr, const char *item);
 int util_uidvalidity (mu_mailbox_t smbox, unsigned long *uidvp);
 
-void util_setio (FILE*, FILE*);
-void util_flush_output (void);
-void util_get_input (mu_stream_t *pstr);
-void util_get_output (mu_stream_t *pstr);
-void util_set_input (mu_stream_t str);
-void util_set_output (mu_stream_t str);
-int util_wait_input (int);
-  
-void util_register_event (int old_state, int new_state,
-			  mu_list_action_t *action, void *data);
-void util_event_remove (void *id);
-void util_run_events (int old_state, int new_state);
   
 int util_is_master (void);
 void util_bye (void);  
@@ -378,13 +402,33 @@ void util_chdir (const char *homedir);
 int is_atom (const char *s);
 int util_isdelim (const char *str);
 int util_trim_nl (char *s, size_t len);
-  
+
+int set_xscript_level (int xlev);
+
 #ifdef WITH_TLS
 int imap4d_init_tls_server (void);
 #endif /* WITH_TLS */
 
-typedef int (*imap4d_auth_handler_fp) (struct imap4d_command *,
-				       char *, char **);
+struct imap4d_auth
+{
+  /* input */
+  struct imap4d_command *command;
+  char *auth_type;
+  /* output */
+  char *username;
+  int response;
+};
+
+enum imap4d_auth_result
+  {
+    imap4d_auth_nosup,
+    imap4d_auth_ok,
+    imap4d_auth_resp,
+    imap4d_auth_fail
+  };
+  
+typedef enum imap4d_auth_result
+          (*imap4d_auth_handler_fp) (struct imap4d_auth *);
   
 extern void auth_add (char *name, imap4d_auth_handler_fp handler);
 extern void auth_remove (char *name);
@@ -400,7 +444,12 @@ extern void auth_gsasl_init (void);
 #else
 # define auth_gsasl_init()
 #endif
-  
+
+/* Quota support */
+void quota_setup (void);
+int quota_check (mu_off_t size);
+void quota_update (mu_off_t size);
+
 #ifdef __cplusplus
 }
 #endif
