@@ -1,5 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001, 2005, 2007, 2008 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2001, 2005, 2007, 2008, 2009, 2010, 2011 Free
+   Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,9 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include "imap4d.h"
 
@@ -48,7 +47,7 @@ status_get_handler (const char *name)
   struct status_table *p;
 
   for (p = status_table; p->name; p++)
-    if (strcasecmp (p->name, name) == 0)
+    if (mu_c_strcasecmp (p->name, name) == 0)
       return p->fun;
   return NULL;
 }
@@ -78,18 +77,20 @@ imap4d_status (struct imap4d_command *command, imap4d_tokbuf_t tok)
   int argc = imap4d_tokbuf_argc (tok);
 
   if (argc < 4)
-    return util_finish (command, RESP_BAD, "Invalid arguments");
+    return io_completion_response (command, RESP_BAD, "Invalid arguments");
   
   name = imap4d_tokbuf_getarg (tok, IMAP4_ARG_1);
 
   mailbox_name = namespace_getfullpath (name, delim, NULL);
 
   if (!mailbox_name)
-    return util_finish (command, RESP_NO, "Error opening mailbox");
+    return io_completion_response (command, RESP_NO, "Error opening mailbox");
 
   /* We may be opening the current mailbox, so make sure the attributes are
      preserved */
+  imap4d_enter_critical ();
   mu_mailbox_sync (mbox);
+  imap4d_leave_critical ();
   
   status = mu_mailbox_create_default (&smbox, mailbox_name);
   if (status == 0)
@@ -104,7 +105,8 @@ imap4d_status (struct imap4d_command *command, imap4d_tokbuf_t tok)
 	  if (item[0] == '(')
 	    {
 	      if (imap4d_tokbuf_getarg (tok, argc - 1)[0] != ')')
-		return util_finish (command, RESP_BAD, "Invalid arguments");
+		return io_completion_response (command, RESP_BAD, 
+		                               "Invalid arguments");
 	      argc--;
 	      i++;
 	    }
@@ -122,11 +124,11 @@ imap4d_status (struct imap4d_command *command, imap4d_tokbuf_t tok)
 		}
 		  
 	      if (count++ == 0)
-		util_send ("* STATUS %s (", name);
+		io_sendf ("* STATUS %s (", name);
 	      else if (!space_sent)
                 {
                   space_sent = 1;
-		  util_send (" ");
+		  io_sendf (" ");
                 }     
 
 	      if (!fun (smbox))
@@ -135,7 +137,7 @@ imap4d_status (struct imap4d_command *command, imap4d_tokbuf_t tok)
 
 	  
 	  if (count > 0)
-	    util_send (")\r\n");
+	    io_sendf (")\n");
 	  mu_mailbox_close (smbox);
 	}
       mu_mailbox_destroy (&smbox);
@@ -145,13 +147,14 @@ imap4d_status (struct imap4d_command *command, imap4d_tokbuf_t tok)
   if (status == 0)
     {
       if (count == 0)
-	return util_finish (command, RESP_BAD, "Too few args (empty list)");
+	return io_completion_response (command, RESP_BAD, 
+	                               "Too few args (empty list)");
       else if (err_msg)
-	return util_finish (command, RESP_BAD, err_msg);
-      return util_finish (command, RESP_OK, "Completed");
+	return io_completion_response (command, RESP_BAD, "%s", err_msg);
+      return io_completion_response (command, RESP_OK, "Completed");
     }
   
-  return util_finish (command, RESP_NO, "Error opening mailbox");
+  return io_completion_response (command, RESP_NO, "Error opening mailbox");
 }
 
 static int
@@ -159,7 +162,7 @@ status_messages (mu_mailbox_t smbox)
 {
   size_t total = 0;
   mu_mailbox_messages_count (smbox, &total);
-  util_send ("MESSAGES %u", total);
+  io_sendf ("MESSAGES %lu", (unsigned long) total);
   return 0;
 }
 
@@ -168,7 +171,7 @@ status_recent (mu_mailbox_t smbox)
 {
   size_t recent = 0;
   mu_mailbox_messages_recent (smbox, &recent);
-  util_send ("RECENT %u", recent);
+  io_sendf ("RECENT %lu", (unsigned long) recent);
   return 0;
 }
 
@@ -177,7 +180,7 @@ status_uidnext (mu_mailbox_t smbox)
 {
   size_t uidnext = 1;
   mu_mailbox_uidnext (smbox, &uidnext);
-  util_send ("UIDNEXT %u", uidnext);
+  io_sendf ("UIDNEXT %lu", (unsigned long) uidnext);
   return 0;
 }
 
@@ -186,7 +189,7 @@ status_uidvalidity (mu_mailbox_t smbox)
 {
   unsigned long uidvalidity = 0;
   util_uidvalidity (smbox, &uidvalidity);
-  util_send ("UIDVALIDITY %lu", uidvalidity);
+  io_sendf ("UIDVALIDITY %lu", uidvalidity);
   return 0;
 }
 
@@ -213,6 +216,6 @@ status_unseen (mu_mailbox_t smbox)
       if (!mu_attribute_is_read (attr))
 	unseen++;
     }
-  util_send ("UNSEEN %d", unseen);
+  io_sendf ("UNSEEN %lu", (unsigned long) unseen);
   return 0;
 }
