@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001, 2005, 2007, 2008,
-   2009 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2001, 2005, 2007, 2008, 2009, 2010, 2011 Free
+   Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,9 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include "imap4d.h"
 
@@ -28,7 +26,7 @@ struct store_parse_closure
   int type;
   int isuid;
   size_t *set;
-  size_t count;
+  int count;
 };
   
 static int
@@ -151,13 +149,13 @@ imap4d_store0 (imap4d_tokbuf_t tok, int isuid, char **ptext)
 	  
 	  if (pclos.ack)
 	    {
-	      util_send ("* %d FETCH (", msgno);
+	      io_sendf ("* %lu FETCH (", (unsigned long) msgno);
 	      
 	      if (isuid)
-		util_send ("UID %lu ", (unsigned long) msgno);
-	      util_send ("FLAGS (");
+		io_sendf ("UID %lu ", (unsigned long) msgno);
+	      io_sendf ("FLAGS (");
 	      util_print_flags (attr);
-	      util_send ("))\r\n");
+	      io_sendf ("))\n");
 	    }
 	  /* Update the flags of uid table.  */
 	  imap4d_sync_flags (pclos.set[i]);
@@ -178,6 +176,6 @@ imap4d_store (struct imap4d_command *command, imap4d_tokbuf_t tok)
   char *err_text;
   
   rc = imap4d_store0 (tok, 0, &err_text);
-  return util_finish (command, rc, "%s", err_text);
+  return io_completion_response (command, rc, "%s", err_text);
 }
 
