@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2005, 2007, 2010 Free Software
+   Copyright (C) 1999, 2000, 2001, 2005, 2007, 2010, 2011 Free Software
    Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
@@ -13,9 +13,8 @@
    Lesser General Public License for more details.
 
    You should have received a copy of the GNU Lesser General
-   Public License along with this library; if not, write to the
-   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301 USA */
+   Public License along with this library.  If not, see 
+   <http://www.gnu.org/licenses/>. */
 
 #ifndef _MAILUTILS_HEADER_H
 #define _MAILUTILS_HEADER_H
@@ -36,7 +35,7 @@ extern "C" {
 #define MU_HEADER_RESENT_FROM               "Resent-From"
 #define MU_HEADER_SUBJECT                   "Subject"
 #define MU_HEADER_SENDER                    "Sender"
-#define MU_HEADER_RESENT_SENDER             "Resent-SENDER"
+#define MU_HEADER_RESENT_SENDER             "Resent-Sender"
 #define MU_HEADER_TO                        "To"
 #define MU_HEADER_RESENT_TO                 "Resent-To"
 #define MU_HEADER_CC                        "Cc"
@@ -55,7 +54,7 @@ extern "C" {
 #define MU_HEADER_STATUS                    "Status"
 #define MU_HEADER_CONTENT_LENGTH            "Content-Length"
 #define MU_HEADER_CONTENT_LANGUAGE          "Content-Language"
-#define MU_HEADER_CONTENT_TRANSFER_ENCODING "Content-transfer-encoding"
+#define MU_HEADER_CONTENT_TRANSFER_ENCODING "Content-Transfer-Encoding"
 #define MU_HEADER_CONTENT_ID                "Content-ID"
 #define MU_HEADER_CONTENT_TYPE              "Content-Type"
 #define MU_HEADER_CONTENT_DESCRIPTION       "Content-Description"
@@ -69,16 +68,17 @@ extern "C" {
 #define MU_HEADER_ENV_SENDER                "X-Envelope-Sender"
 #define MU_HEADER_ENV_DATE                  "X-Envelope-Date"
 #define MU_HEADER_FCC                       "Fcc"
-#define MU_HEADER_DELIVERY_DATE             "Delivery-date"
-#define MU_HEADER_ENVELOPE_TO               "Envelope-to"
+#define MU_HEADER_DELIVERY_DATE             "Delivery-Date"
+#define MU_HEADER_ENVELOPE_TO               "Envelope-To"
 #define MU_HEADER_X_EXPIRE_TIMESTAMP        "X-Expire-Timestamp"
   
 #define MU_HEADER_REPLACE 0x01
 #define MU_HEADER_BEFORE  0x02
 
-extern int mu_header_create (mu_header_t *, const char *, size_t, void *);
-extern void mu_header_destroy (mu_header_t *, void *);
-extern void *mu_header_get_owner (mu_header_t);
+extern int mu_header_create (mu_header_t *, const char *, size_t);
+extern void mu_header_destroy (mu_header_t *);
+extern int mu_header_invalidate (mu_header_t);
+extern int mu_header_clear (mu_header_t header);
 
 extern int mu_header_is_modified (mu_header_t);
 extern int mu_header_clear_modified (mu_header_t);
@@ -106,6 +106,9 @@ extern int mu_header_get_value_n (mu_header_t, const char *, int, char *,
 extern int mu_header_aget_value_n (mu_header_t, const char *, int, char **);
 #define mu_header_aget_value(header, name, pptr) \
   mu_header_aget_value_n (header, name, 1, pptr)
+
+int mu_header_sget_firstof (mu_header_t hdr, char **names,
+			    const char **pval, int *pidx);
   
 /* Get field values as an mu_address_t. */
 extern int mu_header_get_address_n (mu_header_t, const char *,
@@ -149,7 +152,10 @@ extern int mu_header_get_field_value_unfold (mu_header_t header, size_t num,
 extern int mu_header_aget_field_value_unfold (mu_header_t header, size_t num,
 					      char **pvalue);
 
-extern int mu_header_get_stream (mu_header_t, mu_stream_t *);
+extern int mu_header_get_stream (mu_header_t, mu_stream_t *)
+                                  __attribute__ ((deprecated));
+extern int mu_header_get_streamref (mu_header_t, mu_stream_t *);
+  
 /* FIXME: This function does not exist:
    extern int mu_header_set_stream (mu_header_t, mu_stream_t, void *);
 */
@@ -160,7 +166,8 @@ extern int mu_header_get_iterator (mu_header_t, mu_iterator_t *);
   
 
 extern int mu_header_set_fill (mu_header_t,
-      int (*_fill) (mu_header_t, char *, size_t, mu_off_t, size_t *), void *owner);
+			       int (*_fill) (void *data, char **, size_t *),
+			       void *data);
 
   
 #ifdef __cplusplus
