@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002, 2003,
-   2004, 2007  Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2007, 2008, 2009,
+   2010, 2011 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -12,14 +12,16 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.
 
-   You should have received a copy of the GNU Lesser General
-   Public License along with this library; if not, write to the
-   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301 USA */
+   You should have received a copy of the GNU General Public License
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
+#include <mailutils/types.h>
+
+#include <mailutils/acl.h>
 #include <mailutils/address.h>
+#include <mailutils/alloc.h>
+#include <mailutils/assoc.h>
 #include <mailutils/argcv.h>
-#include <mailutils/argp.h>
 #include <mailutils/attribute.h>
 #include <mailutils/auth.h>
 #include <mailutils/body.h>
@@ -31,7 +33,8 @@
 #include <mailutils/folder.h>
 #include <mailutils/header.h>
 #include <mailutils/iterator.h>
-#include <mailutils/libsieve.h>
+#include <mailutils/kwd.h>
+#include <mailutils/sieve.h>
 #include <mailutils/list.h>
 #include <mailutils/locker.h>
 #include <mailutils/mailbox.h>
@@ -41,21 +44,30 @@
 #include <mailutils/mime.h>
 #include <mailutils/monitor.h>
 #include <mailutils/mu_auth.h>
-#include <mailutils/mutil.h>
+#include <mailutils/util.h>
 #include <mailutils/nls.h>
 #include <mailutils/observer.h>
+#include <mailutils/opool.h>
 #include <mailutils/parse822.h>
 #include <mailutils/pop3.h>
+#include <mailutils/progmailer.h>
 #include <mailutils/property.h>
 #include <mailutils/refcount.h>
 #include <mailutils/registrar.h>
 #include <mailutils/stream.h>
 #include <mailutils/tls.h>
-#include <mailutils/types.h>
 #include <mailutils/url.h>
-
-#ifdef MU_COMPAT
-# include <mailutils/compat.h>
-#endif
+#include <mailutils/gocs.h>
+#include <mailutils/version.h>
+#include <mailutils/io.h>
+#include <mailutils/secret.h>
+#include <mailutils/cctype.h>
+#include <mailutils/cstr.h>
+#include <mailutils/wordsplit.h>
+#include <mailutils/log.h>
+#include <mailutils/stdstream.h>
+#include <mailutils/prog.h>
+#include <mailutils/sockaddr.h>
+#include <mailutils/cidr.h>
 
 /* EOF */
