@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2005, 2006,
-   2007 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2005, 2006, 2007, 2010, 2011 Free
+   Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -13,9 +13,8 @@
    Lesser General Public License for more details.
 
    You should have received a copy of the GNU Lesser General
-   Public License along with this library; if not, write to the
-   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301 USA */
+   Public License along with this library.  If not, see 
+   <http://www.gnu.org/licenses/>. */
 
 #ifndef _MAILUTILS_MESSAGE_H
 #define _MAILUTILS_MESSAGE_H
@@ -26,6 +25,25 @@
 extern "C" {
 #endif
 
+#define MU_SCAN_SEEK  0x01
+#define MU_SCAN_SIZE  0x02
+
+struct mu_message_scan
+{
+  int flags;
+  mu_off_t message_start;
+  mu_off_t message_size;
+
+  mu_off_t body_start;
+  mu_off_t body_end;
+  size_t header_lines;
+  size_t body_lines;
+  int attr_flags;
+  unsigned long uidvalidity;
+};
+
+int mu_stream_scan_message (mu_stream_t stream, struct mu_message_scan *sp);
+  
 /* A message is considered to be a container for:
   mu_header_t, mu_body_t, and its mu_attribute_t.  */
 
@@ -34,14 +52,19 @@ extern void mu_message_destroy (mu_message_t *, void *owner);
 
 extern int mu_message_create_copy (mu_message_t *to, mu_message_t from);
 
-extern void * mu_message_get_owner (mu_message_t);
+extern void *mu_message_get_owner (mu_message_t);
+
+#define MU_MSG_ATTRIBUTE_MODIFIED 0x01
+#define MU_MSG_HEADER_MODIFIED    0x02 
+#define MU_MSG_BODY_MODIFIED      0x04
+
 extern int mu_message_is_modified (mu_message_t);
 extern int mu_message_clear_modified (mu_message_t);
 extern int mu_message_get_mailbox (mu_message_t, mu_mailbox_t *);
 extern int mu_message_set_mailbox (mu_message_t, mu_mailbox_t, void *);
 
-extern int mu_message_ref (mu_message_t);
-#define mu_message_unref(msg) mu_message_destroy (&msg, NULL)
+extern void mu_message_ref (mu_message_t);
+extern void mu_message_unref (mu_message_t);
 
 extern int mu_message_get_envelope (mu_message_t, mu_envelope_t *);
 extern int mu_message_set_envelope (mu_message_t, mu_envelope_t, void *);
@@ -52,14 +75,21 @@ extern int mu_message_set_header (mu_message_t, mu_header_t, void *);
 extern int mu_message_get_body (mu_message_t, mu_body_t *);
 extern int mu_message_set_body (mu_message_t, mu_body_t, void *);
 
-extern int mu_message_get_stream (mu_message_t, mu_stream_t *);
-extern int mu_message_set_stream (mu_message_t, mu_stream_t, void *);
+extern int mu_message_get_stream (mu_message_t, mu_stream_t *)
+                                   __attribute__((deprecated));
+extern int mu_message_get_streamref (mu_message_t, mu_stream_t *);
+
+extern int mu_message_set_stream (mu_message_t, mu_stream_t, void *)
+  __attribute__((deprecated));
 
 extern int mu_message_get_attribute (mu_message_t, mu_attribute_t *);
 extern int mu_message_set_attribute (mu_message_t, mu_attribute_t, void *);
 
 extern int mu_message_get_observable (mu_message_t, mu_observable_t *);
 
+extern int mu_message_set_get_stream (mu_message_t,
+				      int (*) (mu_message_t, mu_stream_t *),
+				      void *);
 extern int mu_message_is_multipart (mu_message_t, int *);
 extern int mu_message_set_is_multipart (mu_message_t, 
 					int (*_is_multipart) (mu_message_t,
@@ -72,8 +102,9 @@ extern int mu_message_set_size (mu_message_t,
 				void *owner);
 
 extern int mu_message_lines (mu_message_t, size_t *);
+extern int mu_message_quick_lines (mu_message_t, size_t *);
 extern int mu_message_set_lines (mu_message_t, 
-				 int (*_lines) (mu_message_t, size_t *),
+				 int (*_lines) (mu_message_t, size_t *, int),
 				 void *owner);
 
 extern int mu_message_get_num_parts (mu_message_t, size_t *nparts);
@@ -113,18 +144,56 @@ extern int mu_message_create_attachment (const char *content_type,
 					 const char *filename,
 					 mu_message_t *newmsg);
 extern int mu_message_save_attachment (mu_message_t msg,
-				       const char *filename, void **data);
+				       const char *filename,
+				       mu_mime_io_buffer_t buf);
 extern int mu_message_encapsulate (mu_message_t msg, mu_message_t *newmsg,
-				   void **data);
+				   mu_mime_io_buffer_t buf);
 extern int mu_message_unencapsulate (mu_message_t msg, mu_message_t *newmsg,
-				     void **data);
+				     mu_mime_io_buffer_t buf);
 
+extern int mu_mime_io_buffer_create (mu_mime_io_buffer_t *pinfo);
+extern void mu_mime_io_buffer_destroy (mu_mime_io_buffer_t *pinfo);
+  
+extern int mu_mime_io_buffer_set_charset (mu_mime_io_buffer_t info,
+					  const char *charset);
+extern void mu_mime_io_buffer_sget_charset (mu_mime_io_buffer_t info,
+					    const char **charset);
+extern int mu_mime_io_buffer_aget_charset (mu_mime_io_buffer_t info,
+					   const char **charset);
+
+  
+/* Bit values for *pflags in functions below */
+#define MU_MIMEHDR_MULTILINE 0x01  /* Parameter was multiline */
+#define MU_MIMEHDR_CSINFO    0x02  /* Parameter contains charset/language
+				      info */
+
+extern int mu_mimehdr_get_disp (const char *str, char *buf, size_t bufsz,
+				size_t *retsz);
+extern int mu_mimehdr_aget_disp (const char *str, char **pvalue);
+extern int mu_mimehdr_get_param (const char *str, const char *param,
+				 char *buf, size_t bufsz, size_t *retsz,
+				 int *pflags);
+extern int mu_mimehdr_aget_param (const char *str, const char *param,
+				  char **pval, int *pflags);
+extern int mu_mimehdr_decode_param (const char *value, int csinfo,
+				    const char *charset,
+				    char **pval, char **plang);
+extern int mu_mimehdr_aget_decoded_param (const char *str, const char *param,
+					  const char *charset, 
+					  char **pval, char **plang);
+  
 extern int mu_message_get_attachment_name (mu_message_t, char *name,
-					   size_t bufsz, size_t* sz);
-extern int mu_message_aget_attachment_name (mu_message_t, char **name);
+					   size_t bufsz, size_t* sz,
+					   int *pflags);
+extern int mu_message_aget_attachment_name (mu_message_t, char **name,
+					    int *pflags);
+extern int mu_message_aget_decoded_attachment_name (mu_message_t msg,
+						    const char *charset,
+						    char **name,
+						    char **plang);
 
-extern int mu_message_save_to_mailbox (mu_message_t msg, mu_ticket_t ticket,
-				       mu_debug_t debug, const char *toname);
+extern int mu_message_save_to_mailbox (mu_message_t msg, const char *toname,
+				       int perms);
 
 
 extern int mu_stream_to_message (mu_stream_t instream, mu_message_t *pmsg);
