@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2007 Free Software Foundation, Inc.
+   Copyright (C) 2007, 2008, 2010, 2011 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -12,9 +12,8 @@
    Lesser General Public License for more details.
 
    You should have received a copy of the GNU Lesser General
-   Public License along with this library; if not, write to the
-   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301 USA */
+   Public License along with this library.  If not, see
+   <http://www.gnu.org/licenses/>. */
 
 #ifndef _MAILUTILS_SYSLOG_H
 #define _MAILUTILS_SYSLOG_H
@@ -25,13 +24,17 @@
 extern "C" { 
 #endif
   
+extern int mu_log_syslog;
 extern int mu_log_facility;
 extern char *mu_log_tag;
+extern int mu_log_print_severity;
+extern unsigned mu_log_severity_threshold;
+
 #define MU_LOG_TAG() (mu_log_tag ? mu_log_tag : mu_program_name)
   
-int mu_string_to_syslog_facility (char *str, int *pfacility);
+int mu_string_to_syslog_facility (const char *str, int *pfacility);
 const char *mu_syslog_facility_to_string (int n);
-int mu_string_to_syslog_priority (char *str, int *pprio);
+int mu_string_to_syslog_priority (const char *str, int *pprio);
 const char *mu_syslog_priority_to_string (int n);
 
 #ifdef __cplusplus
