@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2008 Free Software Foundation, Inc.
+   Copyright (C) 2008, 2010, 2011 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,9 +12,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -26,7 +24,16 @@ void
 mu_set_signals (RETSIGTYPE (*handler) (int signo), int *sigv, int sigc)
 {
   int i;
+  struct sigaction act;
 
+  act.sa_flags = 0;
+  sigemptyset (&act.sa_mask);
   for (i = 0; i < sigc; i++)
-    signal (sigv[i], handler);
+    sigaddset (&act.sa_mask, i);
+      
+  for (i = 0; i < sigc; i++)
+    {
+      act.sa_handler = handler;
+      sigaction (sigv[i], &act, NULL);
+    }
 }
