@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
    Copyright (C) 1999, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
-   2010 Free Software Foundation, Inc.
+   2010, 2011 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,26 +13,18 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include <mailutils/types.h>
 
 extern int mu_tcp_wrapper_enable;
 const char *mu_tcp_wrapper_daemon;
-extern int mu_tcp_wrapper_cb_hosts_allow (mu_debug_t debug, void *data,
-					  char *arg);
-extern int mu_tcp_wrapper_cb_hosts_deny (mu_debug_t debug, void *data,
-					 char *arg);
-extern int mu_tcp_wrapper_cb_hosts_allow_syslog (mu_debug_t debug, void *data,
-						 char *arg);
-extern int mu_tcp_wrapper_cb_hosts_deny_syslog (mu_debug_t debug, void *data,
-						char *arg);
 extern int mu_tcpwrapper_access (int fd);
 extern void mu_tcpwrapper_cfg_init (void);
-extern int mu_tcp_wrapper_prefork (int fd, void *data,
-				   struct sockaddr *sa, int salen);
+extern int mu_tcp_wrapper_prefork (int fd, 
+				   struct sockaddr *sa, int salen,
+				   struct mu_srv_config *pconf,
+				   void *data);
 
 #ifdef WITH_LIBWRAP
 # define TCP_WRAPPERS_CONFIG { "tcp-wrappers", mu_cfg_section },
