@@ -1,6 +1,6 @@
 /* utmp.c -- Replacements for {set,get,end}utmp functions
 
-Copyright (C) 2002, 2009, 2010 Free Software Foundation, Inc.
+Copyright (C) 2002, 2009, 2010, 2011 Free Software Foundation, Inc.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License as
@@ -12,10 +12,8 @@ WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.
 
-You should have received a copy of the GNU General Public
-Licensealong with this program; if not, write to the Free
-SoftwareFoundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-MA 02110-1301 USA. */
+You should have received a copy of the GNU General Public License
+along with this program.  If not, see <http://www.gnu.org/licenses/>. */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
