@@ -1,7 +1,6 @@
-/*
-   GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2004, 2006, 2007, 2009, 2010 Free Software Foundation,
-   Inc.
+/* GNU Mailutils -- a suite of utilities for electronic mail
+   Copyright (C) 2004, 2006, 2007, 2009, 2010, 2011 Free Software
+   Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -14,10 +13,8 @@
    Lesser General Public License for more details.
 
    You should have received a copy of the GNU Lesser General
-   Public License along with this library; if not, write to the
-   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301 USA
-*/
+   Public License along with this library.  If not, see
+   <http://www.gnu.org/licenses/>. */
 
 #include <mailutils/cpp/mailbox.h>
 
@@ -163,18 +160,6 @@ MailboxBase :: get_size ()
   return size;
 }
 
-Debug&
-MailboxBase :: get_debug ()
-{
-  mu_debug_t c_dbg;
-
-  int status = mu_mailbox_get_debug (mbox, &c_dbg);
-  if (status)
-    throw Exception ("MailboxBase::get_debug", status);
-
-  return *new Debug (c_dbg);
-}
-
 Folder&
 MailboxBase :: get_folder ()
 {
