@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2006, 2007, 2010 Free Software
+   Copyright (C) 1999, 2000, 2001, 2006, 2007, 2010, 2011 Free Software
    Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
@@ -13,9 +13,8 @@
    Lesser General Public License for more details.
 
    You should have received a copy of the GNU Lesser General
-   Public License along with this library; if not, write to the
-   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301 USA */
+   Public License along with this library.  If not, see
+   <http://www.gnu.org/licenses/>. */
 
 #include "mu_scm.h"
 #include <mailutils/iterator.h>
@@ -137,6 +136,15 @@ mu_scm_is_mailbox (SCM scm)
 /* ************************************************************************* */
 /* Guile primitives */
 
+SCM_DEFINE_PUBLIC (scm_mu_mailbox_p, "mu-mailbox?", 1, 0, 0,
+		   (SCM scm),
+"Return @code{true} if @var{scm} is a Mailutils mailbox.\n")
+#define FUNC_NAME s_scm_mu_mailbox_p
+{
+  return scm_from_bool (mu_scm_is_mailbox (scm));
+}
+#undef FUNC_NAME
+
 SCM_DEFINE_PUBLIC (scm_mu_mail_directory, "mu-mail-directory", 0, 1, 0,
 		   (SCM url), 
 "Do not use this function. Use mu-user-mailbox-url instead.")
@@ -316,7 +324,7 @@ SCM_DEFINE_PUBLIC (scm_mu_mailbox_get_port, "mu-mailbox-get-port", 2, 0, 0,
   SCM_ASSERT (mu_scm_is_mailbox (mbox), mbox, SCM_ARG1, FUNC_NAME);
   SCM_ASSERT (scm_is_string (mode), mode, SCM_ARG2, FUNC_NAME);
   mum = (struct mu_mailbox *) SCM_CDR (mbox);
-  status = mu_mailbox_get_stream (mum->mbox, &stream);
+  status = mu_mailbox_get_streamref (mum->mbox, &stream);
   if (status)
     mu_scm_error (FUNC_NAME, status,
 		  "Cannot get mailbox stream",
