@@ -1,5 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2004, 2005, 2007 Free Software Foundation, Inc.
+   Copyright (C) 2004, 2005, 2007, 2010, 2011 Free Software Foundation,
+   Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -12,9 +13,8 @@
    Lesser General Public License for more details.
 
    You should have received a copy of the GNU Lesser General
-   Public License along with this library; if not, write to the
-   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301 USA */
+   Public License along with this library.  If not, see
+   <http://www.gnu.org/licenses/>. */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -28,12 +28,13 @@
 #include <string.h>
 #include <errno.h>
 
-#include <folder0.h>
-#include <registrar0.h>
+#include <mailutils/sys/folder.h>
+#include <mailutils/sys/registrar.h>
 
 #include <maildir.h>
-#include <mailutils/mutil.h>
-#include <amd.h>
+#include <mailutils/util.h>
+#include <mailutils/url.h>
+#include <mailutils/sys/amd.h>
 
 static int
 _maildir_folder_init (mu_folder_t folder MU_ARG_UNUSED)
@@ -56,54 +57,53 @@ dir_exists (const char *name, const char *suf)
 }
 
 static int
-_maildir_is_scheme (mu_record_t record, const char *url, int flags)
+_maildir_is_scheme (mu_record_t record, mu_url_t url, int flags)
 {
-  const char *path;
-  
-  if (!url || !record->scheme)
-    return 0;
+  if (mu_url_is_scheme (url, record->scheme))
+    return MU_FOLDER_ATTRIBUTE_FILE & flags; 
 
-  if (strncmp (record->scheme, url, strlen (record->scheme)) == 0)
-    return MU_FOLDER_ATTRIBUTE_ALL & flags; 
-
-  if (mu_scheme_autodetect_p (url, &path))
+  if (mu_scheme_autodetect_p (url))
     {
       /* Attemp auto-detection */
+      const char *path;
       struct stat st;
-      int rc = 0;
       
+      if (mu_url_sget_path (url, &path))
+        return 0;
+
       if (stat (path, &st) < 0)
 	return 0; 
 
       if (!S_ISDIR (st.st_mode))
 	return 0;
 
-      rc |= (MU_FOLDER_ATTRIBUTE_DIRECTORY & flags);
-      
       if ((flags & MU_FOLDER_ATTRIBUTE_FILE)
 	  && dir_exists (path, TMPSUF)
 	     && dir_exists (path, CURSUF)
  	     && dir_exists (path, NEWSUF))
-        return rc | MU_FOLDER_ATTRIBUTE_FILE;
+        return MU_FOLDER_ATTRIBUTE_FILE|MU_FOLDER_ATTRIBUTE_DIRECTORY;
     }
   return 0;
 }
 
-/*
-  MAILDIR url
-  maildir:path
-*/
-int
-_maildir_url_init (mu_url_t url)
+static int
+_maildir_list_p (mu_record_t record, const char *name, int flags MU_ARG_UNUSED)
 {
-  return amd_url_init (url, MU_MAILDIR_SCHEME);
+  return strcmp (name, TMPSUF)
+    && strcmp (name, CURSUF) 
+    && strcmp (name, NEWSUF)
+    && !((strlen (name) > 3) &&
+	 (memcmp (name, ".mh", 3) == 0 || memcmp (name, ".mu", 3) == 0));
 }
 
 static struct _mu_record _maildir_record =
 {
   MU_MAILDIR_PRIO,
   MU_MAILDIR_SCHEME,
-  _maildir_url_init, /* Url init.  */
+  MU_RECORD_LOCAL,
+  MU_URL_SCHEME | MU_URL_PATH | MU_URL_PARAM,
+  MU_URL_PATH,
+  mu_url_expand_path, /* Url init.  */
   _mailbox_maildir_init, /* Mailbox init.  */
   NULL, /* Mailer init.  */
   _maildir_folder_init, /* Folder init.  */
@@ -112,12 +112,13 @@ static struct _mu_record _maildir_record =
   NULL, /* _get_url method.  */
   NULL, /* _get_mailbox method.  */
   NULL, /* _get_mailer method.  */
-  NULL  /* _get_folder method.  */
+  NULL, /* _get_folder method.  */
+  _maildir_list_p
 };
 mu_record_t mu_maildir_record = &_maildir_record;
 
 #else
 #include <stdio.h>
-#include <registrar0.h>
+#include <mailutils/sys/registrar.h>
 mu_record_t mu_maildir_record = NULL;
 #endif
