@@ -1,5 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2004, 2005, 2007 Free Software Foundation, Inc.
+   Copyright (C) 2004, 2005, 2007, 2010, 2011 Free Software Foundation,
+   Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -12,9 +13,8 @@
    Lesser General Public License for more details.
 
    You should have received a copy of the GNU Lesser General
-   Public License along with this library; if not, write to the
-   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301 USA */
+   Public License along with this library.  If not, see
+   <http://www.gnu.org/licenses/>. */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -37,8 +37,7 @@
 # include <strings.h>
 #endif
 
-#include <md5.h>
-
+#include <mailutils/md5.h>
 #include <mailutils/body.h>
 #include <mailutils/debug.h>
 #include <mailutils/errno.h>
@@ -52,9 +51,9 @@
 #include <mailutils/url.h>
 #include <mailutils/nntp.h>
 
-#include <folder0.h>
-#include <mailbox0.h>
-#include "nntp0.h"
+#include <mailutils/sys/folder.h>
+#include <mailutils/sys/mailbox.h>
+#include "mailutils/sys/nntp.h"
 
 
 /*  Functions/Methods that implements the mu_mailbox_t API.  */
@@ -92,7 +91,6 @@ _nntp_mailbox_init (mu_mailbox_t mbox)
 {
   m_nntp_t m_nntp;
   int status = 0;
-  size_t name_len = 0;
 
   /* Allocate specifics for nntp data.  */
   m_nntp = mbox->data = calloc (1, sizeof (*m_nntp));
@@ -106,19 +104,19 @@ _nntp_mailbox_init (mu_mailbox_t mbox)
   m_nntp->mailbox = mbox;		/* Back pointer.  */
 
   /* Retrieve the name of the newsgroup from the URL.  */
-  mu_url_get_path (mbox->url, NULL, 0, &name_len);
-  if (name_len == 0)
+  status = mu_url_aget_path (mbox->url, &m_nntp->name);
+  if (status == MU_ERR_NOENT)
     {
-      /* name "INBOX" is the default.  */
-      m_nntp->name = calloc (6, sizeof (char));
-      strcpy (m_nntp->name, "INBOX");
+      m_nntp->name = strdup ("INBOX");
+      if (!m_nntp->name)
+        return ENOMEM;
     }
-  else
+  else if (status)
+    return status;
+  else  
     {
       char *p;
-      m_nntp->name = calloc (name_len + 1, sizeof (char));
-      mu_url_get_path (mbox->url, m_nntp->name, name_len + 1, NULL);
-      p = strchr (m_nntp->name,'/');
+      p = strchr (m_nntp->name, '/');
       if (p)
 	*p = '\0';
     }
@@ -447,12 +445,12 @@ nntp_mailbox_scan (mu_mailbox_t mbox, size_t msgno, size_t *pcount)
     return 0;
   for (i = msgno; i <= count; i++)
     {
-      if (mu_observable_notify (mbox->observable, MU_EVT_MESSAGE_ADD) != 0)
+      size_t tmp = i;
+      if (mu_observable_notify (mbox->observable, MU_EVT_MESSAGE_ADD,
+				&tmp) != 0)
 	break;
-      if (((i +1) % 10) == 0)
-	{
-	  mu_observable_notify (mbox->observable, MU_EVT_MAILBOX_PROGRESS);
-	}
+      if ((i +1) % 10 == 0)
+	mu_observable_notify (mbox->observable, MU_EVT_MAILBOX_PROGRESS, NULL);
     }
   return 0;
 }
