@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2004, 2007 Free Software Foundation, Inc.
+   Copyright (C) 2004, 2007, 2010, 2011 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -12,9 +12,8 @@
    Lesser General Public License for more details.
 
    You should have received a copy of the GNU Lesser General
-   Public License along with this library; if not, write to the
-   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301 USA */
+   Public License along with this library.  If not, see
+   <http://www.gnu.org/licenses/>. */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -24,28 +23,16 @@
 #include <errno.h>
 #include <mailutils/sys/nntp.h>
 
-int
-mu_nntp_set_debug (mu_nntp_t nntp, mu_debug_t debug)
-{
-  if (nntp == NULL)
-    return EINVAL;
-  if (nntp->debug)
-    mu_debug_destroy (&nntp->debug, NULL);
-  nntp->debug = debug;
-  return 0;
-}
-
-/* FIXME: should be a macro */
 int
 mu_nntp_debug_cmd (mu_nntp_t nntp)
 {
-  MU_DEBUG (nntp->debug, MU_DEBUG_PROT, nntp->io.buf);
+  mu_debug (MU_DEBCAT_FOLDER, MU_DEBUG_PROT, ("%s", nntp->io.buf));
   return 0;
 }
 
 int
 mu_nntp_debug_ack (mu_nntp_t nntp)
 {
-  MU_DEBUG1 (nntp->debug, MU_DEBUG_PROT, "%s\n", nntp->ack.buf);
+  mu_debug (MU_DEBCAT_FOLDER, MU_DEBUG_PROT, "%s", nntp->ack.buf);
   return 0;
 }
