@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2004, 2007, 2010 Free Software Foundation, Inc.
+   Copyright (C) 2004, 2007, 2010, 2011 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -12,9 +12,8 @@
    Lesser General Public License for more details.
 
    You should have received a copy of the GNU Lesser General
-   Public License along with this library; if not, write to the
-   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301 USA */
+   Public License along with this library.  If not, see
+   <http://www.gnu.org/licenses/>. */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -41,7 +40,7 @@ mu_nntp_destroy (mu_nntp_t *pnntp)
 
       /* Release the carrier.  */
       if (nntp->carrier)
-	mu_stream_destroy (&nntp->carrier, nntp);
+	mu_stream_destroy (&nntp->carrier);
 
       /* Any posting residue.  */
       if (nntp->post.buf)
