@@ -1,5 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2004, 2005, 2007 Free Software Foundation, Inc.
+   Copyright (C) 2004, 2005, 2007, 2010, 2011 Free Software Foundation,
+   Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -12,9 +13,8 @@
    Lesser General Public License for more details.
 
    You should have received a copy of the GNU Lesser General
-   Public License along with this library; if not, write to the
-   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301 USA */
+   Public License along with this library.  If not, see
+   <http://www.gnu.org/licenses/>. */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -55,7 +55,7 @@ mu_nntp_send (mu_nntp_t nntp)
 	    return ETIMEDOUT;
 	}
 
-      status = mu_stream_write (nntp->carrier, nntp->io.buf, len, 0, &n);
+      status = mu_stream_write (nntp->carrier, nntp->io.buf, len, &n);
       if (n)
 	{
 	  /* Consume what we sent.  */
@@ -88,7 +88,10 @@ mu_nntp_writeline (mu_nntp_t nntp, const char *format, ...)
      let's try to cope.  */
   do
     {
-      len = vsnprintf (nntp->io.buf, nntp->io.len - 1, format, ap);
+      va_list aq;
+      va_copy(aq, ap);
+      len = vsnprintf (nntp->io.buf, nntp->io.len - 1, format, aq);
+      va_end(aq);
       if (len < 0 || len >= (int)nntp->io.len
 	  || !memchr (nntp->io.buf, '\0', len + 1))
 	{
