@@ -1,5 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2003, 2004, 2007 Free Software Foundation, Inc.
+   Copyright (C) 2003, 2004, 2007, 2010, 2011 Free Software Foundation,
+   Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -12,9 +13,8 @@
    Lesser General Public License for more details.
 
    You should have received a copy of the GNU Lesser General
-   Public License along with this library; if not, write to the
-   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301 USA */
+   Public License along with this library.  If not, see
+   <http://www.gnu.org/licenses/>. */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -36,12 +36,17 @@ mu_pop3_set_carrier (mu_pop3_t pop3, mu_stream_t carrier)
     {
       /* Close any old carrier.  */
       mu_pop3_disconnect (pop3);
-      mu_stream_destroy (&pop3->carrier, pop3);
+      mu_stream_destroy (&pop3->carrier);
     }
+  mu_stream_ref (carrier);
   pop3->carrier = carrier;
+  if (MU_POP3_FISSET (pop3, MU_POP3_TRACE))
+    _mu_pop3_trace_enable (pop3);
+  pop3->state = MU_POP3_CONNECT;
   return 0;
 }
 
+/* FIXME: Is it needed? */
 int
 mu_pop3_get_carrier (mu_pop3_t pop3, mu_stream_t *pcarrier)
 {
@@ -51,6 +56,7 @@ mu_pop3_get_carrier (mu_pop3_t pop3, mu_stream_t *pcarrier)
   if (pcarrier == NULL)
     return MU_ERR_OUT_PTR_NULL;
 
+  mu_stream_ref (pop3->carrier);
   *pcarrier = pop3->carrier;
   return 0;
 }
