@@ -1,5 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2007 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2007, 2010, 2011 Free Software
+   Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -12,9 +13,8 @@
    Lesser General Public License for more details.
 
    You should have received a copy of the GNU Lesser General
-   Public License along with this library; if not, write to the
-   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301 USA */
+   Public License along with this library.  If not, see
+   <http://www.gnu.org/licenses/>. */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -28,7 +28,7 @@
 int
 mu_pop3_retr (mu_pop3_t pop3, unsigned int msgno, mu_stream_t *pstream)
 {
-  int status;
+  int status = EINPROGRESS;
 
   if (pop3 == NULL || msgno == 0)
     return EINVAL;
@@ -40,25 +40,20 @@ mu_pop3_retr (mu_pop3_t pop3, unsigned int msgno, mu_stream_t *pstream)
     case MU_POP3_NO_STATE:
       status = mu_pop3_writeline (pop3, "RETR %d\r\n", msgno);
       MU_POP3_CHECK_ERROR (pop3, status);
-      mu_pop3_debug_cmd (pop3);
+      MU_POP3_FCLR (pop3, MU_POP3_ACK);
       pop3->state = MU_POP3_RETR;
 
     case MU_POP3_RETR:
-      status = mu_pop3_send (pop3);
+      status = mu_pop3_response (pop3, NULL);
       MU_POP3_CHECK_EAGAIN (pop3, status);
-      pop3->acknowledge = 0;
-      pop3->state = MU_POP3_RETR_ACK;
-
-    case MU_POP3_RETR_ACK:
-      status = mu_pop3_response (pop3, NULL, 0, NULL);
-      MU_POP3_CHECK_EAGAIN (pop3, status);
-      mu_pop3_debug_ack (pop3);
       MU_POP3_CHECK_OK (pop3);
+      status = mu_pop3_stream_create (pop3, pstream);
+      MU_POP3_CHECK_ERROR (pop3, status);
+      if (mu_pop3_trace_mask (pop3, MU_POP3_TRACE_QRY, MU_XSCRIPT_PAYLOAD))
+	_mu_pop3_xscript_level (pop3, MU_XSCRIPT_PAYLOAD);
       pop3->state = MU_POP3_RETR_RX;
 
     case MU_POP3_RETR_RX:
-      status = mu_pop3_stream_create (pop3, pstream);
-      MU_POP3_CHECK_ERROR (pop3, status);
       break;
 
       /* They must deal with the error first by reopening.  */
