@@ -1,5 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2007 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2007, 2009, 2010, 2011 Free Software
+   Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -12,23 +13,24 @@
    Lesser General Public License for more details.
 
    You should have received a copy of the GNU Lesser General
-   Public License along with this library; if not, write to the
-   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301 USA */
+   Public License along with this library.  If not, see
+   <http://www.gnu.org/licenses/>. */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
 
+#include <stdio.h>
 #include <string.h>
 #include <errno.h>
 #include <mailutils/sys/pop3.h>
 
 int
-mu_pop3_stat (mu_pop3_t pop3, unsigned *msg_count, size_t *size)
+mu_pop3_stat (mu_pop3_t pop3, size_t *msg_count, mu_off_t *size)
 {
   int status;
-
+  unsigned long lv, count;
+  
   if (pop3 == NULL || msg_count == NULL)
     return EINVAL;
   if (size == NULL)
@@ -39,26 +41,22 @@ mu_pop3_stat (mu_pop3_t pop3, unsigned *msg_count, size_t *size)
     case MU_POP3_NO_STATE:
       status = mu_pop3_writeline (pop3, "STAT\r\n");
       MU_POP3_CHECK_ERROR (pop3, status);
-      mu_pop3_debug_cmd (pop3);
+      MU_POP3_FCLR (pop3, MU_POP3_ACK);
       pop3->state = MU_POP3_STAT;
 
     case MU_POP3_STAT:
-      status = mu_pop3_send (pop3);
+      status = mu_pop3_response (pop3, NULL);
       MU_POP3_CHECK_EAGAIN (pop3, status);
-      pop3->acknowledge = 0;
-      pop3->state = MU_POP3_STAT_ACK;
-
-    case MU_POP3_STAT_ACK:
-      status = mu_pop3_response (pop3, NULL, 0, NULL);
-      MU_POP3_CHECK_EAGAIN (pop3, status);
-      mu_pop3_debug_ack (pop3);
       MU_POP3_CHECK_OK (pop3);
       pop3->state = MU_POP3_NO_STATE;
 
       /* Parse the answer.  */
       *msg_count = 0;
-      *size = 0;
-      sscanf (pop3->ack.buf, "+OK %d %d", msg_count, size);
+      lv = 0;
+      /* FIXME: Error checking */
+      sscanf (pop3->ackbuf, "+OK %lu %lu", &count, &lv);
+      *msg_count = count;
+      *size = lv;
       break;
 
       /* They must deal with the error first by reopening.  */
