@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002, 2005, 2007, 2008, 2009, 2010
-   Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2002, 2005, 2007, 2008, 2009, 2010,
+   2011 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,101 +13,12 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 /* ".forward" support for GNU Maidag */
 
 #include "maidag.h"
 
-/* Functions for checking file mode of .forward and its directory.
-   Each of these checks certain bits and returns 0 if they are OK
-   and non-0 otherwise. */
-
-static int
-check_iwgrp (struct stat *filest, struct stat *dirst)
-{
-  return filest->st_mode & S_IWGRP;
-}
-
-static int
-check_iwoth (struct stat *filest, struct stat *dirst)
-{
-  return filest->st_mode & S_IWOTH;
-}
-
-static int
-check_linked_wrdir (struct stat *filest, struct stat *dirst)
-{
-  return (filest->st_mode & S_IFLNK) && (dirst->st_mode & (S_IWGRP | S_IWOTH));
-}
-
-static int
-check_dir_iwgrp (struct stat *filest, struct stat *dirst)
-{
-  return dirst->st_mode & S_IWGRP;
-}
-
-static int
-check_dir_iwoth (struct stat *filest, struct stat *dirst)
-{
-  return dirst->st_mode & S_IWOTH;
-}
-
-/* The table of permission checkers below has this type: */
-struct perm_checker
-{
-  int flag;              /* FWD_ flag that enables this entry */
-  char *descr;           /* Textual description to use if FUN returns !0 */
-  int (*fun) (struct stat *filest, struct stat *dirst); /* Checker function */
-};
-
-static struct perm_checker perm_check_tab[] = {
-  { FWD_IWGRP, N_("group writable forward file"), check_iwgrp },
-  { FWD_IWOTH, N_("world writable forward file"), check_iwoth },
-  { FWD_LINK, N_("linked forward file in writable dir"), check_linked_wrdir },
-  { FWD_DIR_IWGRP, N_("forward file in group writable directory"),
-    check_dir_iwgrp },
-  { FWD_DIR_IWOTH, N_("forward file in world writable directory"),
-    check_dir_iwoth },
-  { 0 }
-};
-
-/* Check if the forwrd file FILENAME has right permissions and file mode.
-   DIRST describes the directory holding the file, AUTH gives current user
-   authority. */
-int
-check_forward_permissions (const char *filename, struct stat *dirst,
-			   struct mu_auth_data *auth)
-{
-  struct stat st;
-  
-  if (stat (filename, &st) == 0)
-    {
-      int i;
-      
-      if (auth->uid != st.st_uid)
-	{
-	  mu_error (_("%s not owned by %s"), filename, auth->name);
-	  return 1;
-	}
-      for (i = 0; perm_check_tab[i].flag; i++)
-	if ((forward_file_checks & perm_check_tab[i].flag)
-	    && perm_check_tab[i].fun (&st, dirst))
-	  {
-	    mu_error ("%s: %s", filename, gettext (perm_check_tab[i].descr));
-	    return 1;
-	  }
-      return 0;
-    }
-  else if (errno != ENOENT)
-    mu_error (_("%s: cannot stat file: %s"),
-	      filename, mu_strerror (errno));
-  return 1;
-}
-
-
 /* Auxiliary functions */
 
 /* Forward message MSG to given EMAIL, using MAILER and sender address FROM */
@@ -203,22 +114,23 @@ create_from_address (mu_message_t msg, mu_address_t *pfrom)
 enum maidag_forward_result
 process_forward (mu_message_t msg, char *filename, const char *myname)
 {
-  FILE *fp;
-  size_t size = 0;
+  int rc;
+  mu_stream_t file;
+  size_t size = 0, n;
   char *buf = NULL;
   enum maidag_forward_result result = maidag_forward_ok;
   mu_mailer_t mailer = NULL;
   mu_address_t from = NULL;
-  
-  fp = fopen (filename, "r");
-  if (!fp)
+
+  rc = mu_file_stream_create (&file, filename, MU_STREAM_READ);
+  if (rc)
     {
       mu_error (_("%s: cannot open forward file: %s"),
-		filename, mu_strerror (errno));
+		filename, mu_strerror (rc));
       return maidag_forward_error;
     }
 
-  while (getline (&buf, &size, fp) > 0)
+  while (mu_stream_getline (file, &buf, &size, &n) == 0 && n > 0)
     {
       char *p;
 
@@ -243,7 +155,7 @@ process_forward (mu_message_t msg, char *filename, const char *myname)
 		  if (result == maidag_forward_ok)
 		    result = maidag_forward_metoo;
 		}
-	      else if (deliver (msg, p, NULL))
+	      else if (deliver_to_user (msg, p, NULL))
 		result = maidag_forward_error;
 	    }
 	}
@@ -256,10 +168,13 @@ process_forward (mu_message_t msg, char *filename, const char *myname)
       mu_mailer_destroy (&mailer);
     }
   free (buf);
-  fclose (fp);
+  mu_stream_destroy (&file);
   return result;
 }
 
+
+static mu_list_t idlist;
+
 /* Check if the forward file FWFILE for user given by AUTH exists, and if
    so, use to to forward message MSG. */
 enum maidag_forward_result
@@ -268,27 +183,46 @@ maidag_forward (mu_message_t msg, struct mu_auth_data *auth, char *fwfile)
   struct stat st;
   char *filename;
   enum maidag_forward_result result = maidag_forward_none;
+  int rc;
   
-  if (stat (auth->dir, &st))
+  if (fwfile[0] != '/')
     {
-      if (errno == ENOENT)
-	/* FIXME: a warning, maybe? */;
-      else if (!S_ISDIR (st.st_mode))
-	mu_error (_("%s: not a directory"), auth->dir);
-      else
-	mu_error (_("%s: cannot stat directory: %s"),
-		  auth->dir, mu_strerror (errno));
-      return maidag_forward_none;
+      if (stat (auth->dir, &st))
+	{
+	  if (errno == ENOENT)
+	    /* FIXME: a warning, maybe? */;
+	  else if (!S_ISDIR (st.st_mode))
+	    mu_error (_("%s: not a directory"), auth->dir);
+	  else
+	    mu_error (_("%s: cannot stat directory: %s"),
+		      auth->dir, mu_strerror (errno));
+	  return maidag_forward_none;
+	}
+      filename = mu_make_file_name (auth->dir, fwfile);
     }
-  asprintf (&filename, "%s/%s", auth->dir, fwfile);
+  else
+    filename = strdup (fwfile);
+  
   if (!filename)
     {
       mu_error ("%s", mu_strerror (errno));
       return maidag_forward_error;
     }
 
-  if (check_forward_permissions (filename, &st, auth) == 0)
+  if (!idlist)
+    mu_list_create (&idlist);
+
+  rc = mu_file_safety_check (filename, forward_file_checks,
+			     auth->uid, idlist);
+  if (rc == 0)
     result = process_forward (msg, filename, auth->name);
+  else if (rc == MU_ERR_EXISTS)
+    mu_diag_output (MU_DIAG_NOTICE,
+		    _("skipping forward file %s: already processed"),
+		    filename);
+  else
+    mu_error (_("ignoring forward file %s: %s"),
+	      filename, mu_strerror (rc));
   
   free (filename);
   return result;
