@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002, 2004,
-   2005, 2007 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2002, 2004, 2005, 2007, 2009, 2010,
+   2011 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,9 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include "maidag.h"
 
@@ -62,77 +60,99 @@ get_size (char *str, mu_off_t *size, char **endp)
 int
 fail_retrieve_quota (char *name, mu_off_t *quota)
 {
-  mu_error (_("No quota retrieving mechanism"));
+  mu_error (_("no quota retrieving mechanism"));
   return RETR_FAILURE;
 }
 
-#ifdef USE_DBM
+#ifdef ENABLE_DBM
 int
 dbm_retrieve_quota (char *name, mu_off_t *quota)
 {
-  DBM_FILE db;
-  DBM_DATUM named, contentd;
+  mu_dbm_file_t db;
+  struct mu_dbm_datum  named, contentd;
   char buffer[64];
   int unlimited = 0;
   int rc;
 
   if (!quotadbname)
     return RETR_FAILURE;
-  
-  if (mu_dbm_open (quotadbname, &db, MU_STREAM_READ, 0600))
+
+  rc = mu_dbm_create (quotadbname, &db);
+  if (rc)
     {
-      mu_error (_("Cannot open %s: %s"), quotadbname, mu_strerror (errno));
+      mu_error (_("unable to create quota db"));
+      return RETR_FAILURE;
+    }
+
+  rc = mu_dbm_safety_check (db);
+  if (rc && rc != ENOENT)
+    {
+      mu_error (_("quota db fails safety check: %s"),
+		mu_strerror (rc));
+      mu_dbm_destroy (&db);
+      return RETR_FAILURE;
+    }
+
+  rc = mu_dbm_open (db, MU_STREAM_READ, 0600);
+  if (rc)
+    {
+      mu_error (_("cannot open file %s: %s"),
+		quotadbname, mu_strerror (rc));
+      mu_dbm_destroy (&db);
       return RETR_FAILURE;
     }
   
   memset (&named, 0, sizeof named);
   memset (&contentd, 0, sizeof contentd);
-  MU_DATUM_PTR (named) = name;
-  MU_DATUM_SIZE (named) = strlen (name);
-  rc = mu_dbm_fetch (db, named, &contentd);
-  if (rc || !MU_DATUM_PTR (contentd))
+  named.mu_dptr = name;
+  named.mu_dsize = strlen (name);
+  rc = mu_dbm_fetch (db, &named, &contentd);
+  if (rc)
     {
       /* User not in database, try default quota */
-      memset (&named, 0, sizeof named);
-      MU_DATUM_PTR (named) = "DEFAULT";
-      MU_DATUM_SIZE (named) = strlen ("DEFAULT");
-      rc = mu_dbm_fetch (db, named, &contentd);
-      if (rc)
+      mu_dbm_datum_free (&named);
+      named.mu_dptr = "DEFAULT";
+      named.mu_dsize = strlen ("DEFAULT");
+      rc = mu_dbm_fetch (db, &named, &contentd);
+      if (rc == MU_ERR_NOENT)
 	{
-	  /*mu_error (_("can't fetch data: %s"), strerror (rc));*/
+	  mu_dbm_destroy (&db);
+	  return RETR_FAILURE;
+	}
+      else if (rc)
+	{
+	  mu_error (_("can't fetch data: %s"), mu_dbm_strerror (db));
+	  mu_dbm_destroy (&db);
 	  return RETR_FAILURE;
 	}
-      if (!MU_DATUM_PTR (contentd))
-	return RETR_FAILURE;
     }
 
-  if (strncasecmp("none",
-		  MU_DATUM_PTR (contentd),
-		  MU_DATUM_SIZE (contentd)) == 0) 
+  if (mu_c_strncasecmp("none", contentd.mu_dptr, contentd.mu_dsize) == 0) 
       unlimited = 1;
-  else if (MU_DATUM_SIZE (contentd) > sizeof(buffer)-1)
+  else if (contentd.mu_dsize > sizeof (buffer) - 1)
     {
-      mu_error (ngettext ("Mailbox quota for `%s' is too big: %d digit",
-			  "Mailbox quota for `%s' is too big: %d digits",
-			  MU_DATUM_SIZE (contentd)),
-		name, MU_DATUM_SIZE (contentd));
+      mu_error (ngettext ("mailbox quota for `%s' is too big: %d digit",
+			  "mailbox quota for `%s' is too big: %d digits",
+			  contentd.mu_dsize),
+		name, contentd.mu_dsize);
       *quota = groupquota;
     }
   else
     {
       char *p;
 		
-      strncpy(buffer, MU_DATUM_PTR (contentd), MU_DATUM_SIZE (contentd));
-      buffer[MU_DATUM_SIZE (contentd)] = 0;
+      strncpy (buffer, contentd.mu_dptr, contentd.mu_dsize);
+      buffer[contentd.mu_dsize] = 0;
       *quota = strtoul (buffer, &p, 0);
       if (get_size (buffer, quota, &p))
 	{
-	  mu_error (_("Bogus mailbox quota for `%s' (near `%s')"), name, p);
+	  mu_error (_("bogus mailbox quota for `%s' (near `%s')"), name, p);
 	  *quota = groupquota;
 	}
     }
-
-  mu_dbm_close (db);
+  
+  mu_dbm_datum_free (&contentd);
+  mu_dbm_destroy (&db);
   
   return unlimited ? RETR_UNLIMITED : RETR_OK;
 }
@@ -145,8 +165,8 @@ dbm_retrieve_quota (char *name, mu_off_t *quota)
 #ifdef USE_SQL
 #include <mailutils/sql.h>
 
-/* FIXME: defined in auth/sql.c */
-#include <auth/sql.h>
+/* FIXME: defined in libmu_auth/sql.c */
+#include <libmu_auth/sql.h>
 
 int
 sql_retrieve_quota (char *name, mu_off_t *quota)
@@ -194,7 +214,7 @@ sql_retrieve_quota (char *name, mu_off_t *quota)
   
   if (status)
     {
-      mu_error (_("SQL Query failed: %s"),
+      mu_error (_("SQL query failed: %s"),
 		(status == MU_ERR_SQL) ?  mu_sql_strerror (conn) :
 	 	                          mu_strerror (status));
       mu_sql_connection_destroy (&conn);
@@ -205,7 +225,7 @@ sql_retrieve_quota (char *name, mu_off_t *quota)
 
   if (status)
     {
-      mu_error (_("Cannot store SQL result: %s"),
+      mu_error (_("cannot store SQL result: %s"),
 		(status == MU_ERR_SQL) ?  mu_sql_strerror (conn) :
 	 	                          mu_strerror (status));
       mu_sql_connection_destroy (&conn);
@@ -224,12 +244,12 @@ sql_retrieve_quota (char *name, mu_off_t *quota)
       status = mu_sql_get_column (conn, 0, 0, &tmp);
       if (status)
 	{
-	  mu_error (_("Cannot retrieve mailbox quota from SQL: %s"),
+	  mu_error (_("cannot retrieve mailbox quota from SQL: %s"),
 		    (status == MU_ERR_SQL) ?  mu_sql_strerror (conn) :
 		    mu_strerror (status));
 	  rc = RETR_FAILURE;
 	}
-      else if (tmp == NULL || tmp[0] == 0 || strcasecmp (tmp, "none") == 0)
+      else if (tmp == NULL || tmp[0] == 0 || mu_c_strcasecmp (tmp, "none") == 0)
 	rc = RETR_UNLIMITED;
       else
 	{
@@ -237,7 +257,7 @@ sql_retrieve_quota (char *name, mu_off_t *quota)
 	  
 	  if (get_size (tmp, quota, &p))
 	    {
-	      mu_error (_("Bogus mailbox quota for `%s' (near `%s')"),
+	      mu_error (_("bogus mailbox quota for `%s' (near `%s')"),
 			name, p);
 	      *quota = groupquota;
 	    }
