@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001, 2002, 2003, 2005, 
-   2007 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2001, 2002, 2003, 2005, 2007, 2010, 2011 Free
+   Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,9 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include "mail.h"
 
@@ -40,7 +38,7 @@ mail_delete (int argc, char **argv)
   int rc = util_foreach_msg (argc, argv, MSG_NODELETED|MSG_SILENT,
 			     mail_delete_msg, NULL);
 
-  if (util_getenv (NULL, "autoprint", Mail_env_boolean, 0) == 0)
+  if (mailvar_get (NULL, "autoprint", mailvar_type_boolean, 0) == 0)
     util_do_command("print");
 
   return rc;
