@@ -1,5 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001, 2002, 2007 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2001, 2002, 2007, 2010, 2011 Free Software
+   Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,9 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include "mail.h"
 
@@ -53,7 +52,7 @@ _cond_push(int val)
 
   if (!_cond_stack)
     {
-      util_error(_("Not enough memory"));
+      mu_error(_("Not enough memory"));
       exit (EXIT_FAILURE);
     }
   _cond_stack[_cond_level++] = val;
@@ -64,7 +63,7 @@ _cond_pop()
 {
   if (_cond_level == 0)
     {
-      util_error(_("Internal error: condition stack underflow"));
+      mu_error(_("Internal error: condition stack underflow"));
       abort();
     }
   return _cond_stack[--_cond_level];
@@ -87,13 +86,13 @@ mail_if (int argc, char **argv)
   if (argc != 2)
     {
       /* TRANSLATORS: 'if' is the function name. Do not translate it */
-      util_error(_("if requires an argument: s | r | t"));
+      mu_error(_("if requires an argument: s | r | t"));
       return 1;
     }
 
   if (argv[1][1] != 0)
     {
-      util_error(_("Valid if arguments are: s | r | t"));
+      mu_error(_("Valid if arguments are: s | r | t"));
       return 1;
     }
 
@@ -117,7 +116,7 @@ mail_if (int argc, char **argv)
 	  cond = isatty (fileno (stdout));
 	  break;
 	default:
-	  util_error(_("Valid if arguments are: s | r | t"));
+	  mu_error(_("Valid if arguments are: s | r | t"));
 	  return 1;
 	}
     }
@@ -134,7 +133,7 @@ mail_else (int argc MU_ARG_UNUSED, char **argv MU_ARG_UNUSED)
   if (_cond_level == 0)
     {
       /* TRANSLATORS: 'else' and 'if' are function names. Do not translate them */
-      util_error(_("else without matching if"));
+      mu_error(_("else without matching if"));
       return 1;
     }
   cond = _cond_pop();
@@ -150,7 +149,7 @@ mail_endif (int argc MU_ARG_UNUSED, char **argv MU_ARG_UNUSED)
   if (_cond_level == 0)
     {
       /* TRANSLATORS: 'endif' and 'if' are function names. Do not translate them */
-      util_error(_("endif without matching if"));
+      mu_error (_("endif without matching if"));
       return 1;
     }
   _cond_pop();
