@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001, 2002, 2003, 2004, 
-   2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
+   2009, 2010, 2011 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,9 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include "mail.h"
 #include <sys/types.h>
@@ -23,7 +21,7 @@
 #include <fcntl.h>
 
 static int isfilename (const char *);
-static void msg_to_pipe (const char *cmd, mu_message_t msg);
+static int msg_to_pipe (const char *cmd, mu_message_t msg);
 
 
 /* Additional message headers */
@@ -54,7 +52,7 @@ list_headers (void *item, void *data)
 
   if (!name || strcmp (name, hp->name) == 0)
     {
-      printf ("%s: %s\n", hp->name, hp->value);
+      mu_printf ("%s: %s\n", hp->name, hp->value);
     }
   return 0;
 }
@@ -69,7 +67,7 @@ add_header (char *name, char *value, int mode)
       int rc = mu_list_create (&add_header_list);
       if (rc)
 	{
-	  util_error (_("Cannot create header list: %s"), mu_strerror (rc));
+	  mu_error (_("Cannot create header list: %s"), mu_strerror (rc));
 	  exit (1);
 	}
     }
@@ -91,7 +89,7 @@ send_append_header (char *text)
   p = strchr (text, ':');
   if (!p)
     {
-      util_error (_("Invalid header: %s"), text);
+      mu_error (_("Invalid header: %s"), text);
       return;
     }
   len = p - text;
@@ -223,9 +221,9 @@ compose_header_set (compose_env_t * env, const char *name,
     return EINVAL;
 
   if (!env->header
-      && (status = mu_header_create (&env->header, NULL, 0, NULL)) != 0)
+      && (status = mu_header_create (&env->header, NULL, 0)) != 0)
     {
-      util_error (_("Cannot create header: %s"), mu_strerror (status));
+      mu_error (_("Cannot create header: %s"), mu_strerror (status));
       return status;
     }
 
@@ -293,40 +291,49 @@ compose_header_get (compose_env_t * env, char *name, char *defval)
 }
 
 void
-compose_destroy (compose_env_t * env)
+compose_destroy (compose_env_t *env)
 {
-  mu_header_destroy (&env->header, NULL);
+  mu_header_destroy (&env->header);
   free (env->outfiles);
 }
 
 static int
-fill_body (mu_message_t msg, FILE *file)
+fill_body (mu_message_t msg, mu_stream_t instr)
 {
+  int rc;
   mu_body_t body = NULL;
   mu_stream_t stream = NULL;
-  off_t offset = 0;
-  char *buf = NULL;
-  size_t n = 0;
-  mu_message_get_body (msg, &body);
-  mu_body_get_stream (body, &stream);
-
-  while (getline (&buf, &n, file) >= 0)
+  mu_off_t n;
+  
+  rc = mu_message_get_body (msg, &body);
+  if (rc)
     {
-      size_t len = strlen (buf);
-      mu_stream_write (stream, buf, len, offset, &n);
-      offset += len;
+      mu_error (_("cannot get message body: %s"), mu_strerror (rc));
+      return 1;
+    }
+  rc = mu_body_get_streamref (body, &stream);
+  if (rc)
+    {
+      mu_error (_("cannot get body: %s"), mu_strerror (rc));
+      return 1;
     }
-	    
-  if (buf)
-    free (buf);
 
-  if (offset == 0)
+  rc = mu_stream_copy (stream, instr, 0, &n);
+  mu_stream_destroy (&stream);
+
+  if (rc)
+    {
+      mu_error (_("cannot copy temporary stream: %s"), mu_strerror (rc));
+      return 1;
+    }
+  
+  if (n == 0)
     {
       if (mailvar_get (NULL, "nullbody", mailvar_type_boolean, 0) == 0)
 	{
 	  char *str;
 	  if (mailvar_get (&str, "nullbodymsg", mailvar_type_string, 0) == 0)
-	    util_error ("%s\n", _(str));
+	    mu_error ("%s", _(str));
 	}
       else
 	return 1;
@@ -335,6 +342,80 @@ fill_body (mu_message_t msg, FILE *file)
   return 0;
 }
 
+static int
+save_dead_message (compose_env_t *env)
+{
+  if (mailvar_get (NULL, "save", mailvar_type_boolean, 0) == 0)
+    {
+      mu_stream_t dead_letter;
+      int rc;
+      const char *name = getenv ("DEAD");
+
+      /* FIXME: Use MU_STREAM_APPEND if appenddeadletter, instead of the
+	 stream manipulations below */
+      rc = mu_file_stream_create (&dead_letter, name, MU_STREAM_WRITE);
+      if (rc)
+	{
+	  mu_error (_("Cannot open file %s: %s"), name, strerror (rc));
+	  return 1;
+	}
+      if (mailvar_get (NULL, "appenddeadletter",
+		       mailvar_type_boolean, 0) == 0)
+	mu_stream_seek (dead_letter, 0, MU_SEEK_END, NULL);
+      else
+	mu_stream_truncate (dead_letter, 0);
+
+      mu_stream_seek (env->compstr, 0, MU_SEEK_SET, NULL);
+      mu_stream_copy (dead_letter, env->compstr, 0, NULL);
+      mu_stream_destroy (&dead_letter);
+    }
+  return 0;
+}
+
+static int
+send_message (mu_message_t msg)
+{
+  char *sendmail;
+  int status;
+  
+  if (mailvar_get (&sendmail, "sendmail", mailvar_type_string, 0) == 0)
+    {
+      if (sendmail[0] == '/')
+	status = msg_to_pipe (sendmail, msg);
+      else
+	{
+	  mu_mailer_t mailer;
+	  
+	  status = mu_mailer_create (&mailer, sendmail);
+	  if (status == 0)
+	    {
+	      if (mailvar_get (NULL, "verbose", mailvar_type_boolean, 0) == 0)
+		{
+		  mu_debug_set_category_level (MU_DEBCAT_MAILER,
+				      MU_DEBUG_LEVEL_UPTO (MU_DEBUG_PROT));
+		}
+	      status = mu_mailer_open (mailer, MU_STREAM_RDWR);
+	      if (status == 0)
+		{
+		  status = mu_mailer_send_message (mailer, msg, NULL, NULL);
+		  mu_mailer_close (mailer);
+		}
+	      else
+		mu_error (_("Cannot open mailer: %s"), mu_strerror (status));
+	      mu_mailer_destroy (&mailer);
+	    }
+	  else
+	    mu_error (_("Cannot create mailer: %s"),
+			mu_strerror (status));
+	}
+    }
+  else
+    {
+      mu_error (_("Variable sendmail not set: no mailer"));
+      status = ENOSYS;
+    }
+  return status;
+}
 
 /* mail_send0(): shared between mail_send() and mail_reply();
 
@@ -352,29 +433,22 @@ fill_body (mu_message_t msg, FILE *file)
    addresses on the command line and message contents on standard input. */
 
 int
-mail_send0 (compose_env_t * env, int save_to)
+mail_send0 (compose_env_t *env, int save_to)
 {
   int done = 0;
-  int fd;
-  char *filename;
-  FILE *file;
+  int rc;
   char *savefile = NULL;
   int int_cnt;
   char *escape;
 
-  fd = mu_tempfile (NULL, &filename);
-  if (fd == -1)
+  /* Prepare environment */
+  rc = mu_temp_file_stream_create (&env->compstr, NULL, 0);
+  if (rc)
     {
-      util_error (_("Cannot open temporary file"));
+      mu_error (_("Cannot open temporary file: %s"), mu_strerror (rc));
       return 1;
     }
 
-  /* Prepare environment */
-  env = env;
-  env->filename = filename;
-  env->file = fdopen (fd, "w+");
-  env->ofile = ofile;
-
   ml_clear_interrupt ();
   int_cnt = 0;
   while (!done)
@@ -386,7 +460,7 @@ mail_send0 (compose_env_t * env, int save_to)
 	{
 	  if (mailvar_get (NULL, "ignore", mailvar_type_boolean, 0) == 0)
 	    {
-	      fprintf (stdout, "@\n");
+	      mu_printf ("@\n");
 	    }
 	  else
 	    {
@@ -394,7 +468,7 @@ mail_send0 (compose_env_t * env, int save_to)
 		free (buf);
 	      if (++int_cnt == 2)
 		break;
-	      util_error (_("\n(Interrupt -- one more to kill letter)"));
+	      mu_error (_("\n(Interrupt -- one more to kill letter)"));
 	    }
 	  continue;
 	}
@@ -404,7 +478,7 @@ mail_send0 (compose_env_t * env, int save_to)
 	  if (interactive 
 	      && mailvar_get (NULL, "ignoreeof", mailvar_type_boolean, 0) == 0)
 	    {
-	      util_error (mailvar_get (NULL, "dot", mailvar_type_boolean, 0) == 0 ?
+	      mu_error (mailvar_get (NULL, "dot", mailvar_type_boolean, 0) == 0 ?
 			  _("Use \".\" to terminate letter.") :
 			  _("Use \"~.\" to terminate letter."));
 	      continue;
@@ -422,7 +496,7 @@ mail_send0 (compose_env_t * env, int save_to)
 	       && buf[0] == escape[0])
 	{
 	  if (buf[1] == buf[0])
-	    fprintf (env->file, "%s\n", buf + 1);
+	    mu_stream_printf (env->compstr, "%s\n", buf + 1);
 	  else if (buf[1] == '.')
 	    done = 1;
 	  else if (buf[1] == 'x')
@@ -432,77 +506,47 @@ mail_send0 (compose_env_t * env, int save_to)
 	    }
 	  else
 	    {
-	      int argc;
-	      char **argv;
+	      struct mu_wordsplit ws;
 	      int status;
 
-	      ofile = env->file;
-
-	      if (mu_argcv_get (buf + 1, "", NULL, &argc, &argv) == 0)
+	      if (mu_wordsplit (buf + 1, &ws, MU_WRDSF_DEFFLAGS) == 0)
 		{
-		  if (argc > 0)
+		  if (ws.ws_wordc > 0)
 		    {
 		      const struct mail_escape_entry *entry = 
-			            mail_find_escape (argv[0]);
+			mail_find_escape (ws.ws_wordv[0]);
 
 		      if (entry)
-			status = (*entry->escfunc) (argc, argv, env);
+			status = (*entry->escfunc) (ws.ws_wordc, ws.ws_wordv,
+						    env);
 		      else
-			util_error (_("Unknown escape %s"), argv[0]);
+			mu_error (_("Unknown escape %s"), ws.ws_wordv[0]);
 		    }
 		  else
-		    util_error (_("Unfinished escape"));
+		    mu_error (_("Unfinished escape"));
+		  mu_wordsplit_free (&ws);
 		}
 	      else
 		{
-		  util_error (_("Cannot parse escape sequence"));
+		  mu_error (_("Cannot parse escape sequence: %s"),
+			      mu_wordsplit_strerror (&ws));
 		}
-	      mu_argcv_free (argc, argv);
-
-	      ofile = env->ofile;
 	    }
 	}
       else
-	fprintf (env->file, "%s\n", buf);
-      fflush (env->file);
+	mu_stream_printf (env->compstr, "%s\n", buf);
+      mu_stream_flush (env->compstr);
       free (buf);
     }
 
-  /* If interrupted dump the file to dead.letter.  */
+  /* If interrupted, dump the file to dead.letter.  */
   if (int_cnt)
     {
-      if (mailvar_get (NULL, "save", mailvar_type_boolean, 0) == 0)
-	{
-	  FILE *fp = fopen (getenv ("DEAD"),
-			    mailvar_get (NULL, "appenddeadletter",
-					 mailvar_type_boolean, 0) == 0 ?
-			    "a" : "w");
-
-	  if (!fp)
-	    {
-	      util_error (_("Cannot open file %s: %s"), getenv ("DEAD"),
-			  strerror (errno));
-	    }
-	  else
-	    {
-	      char *buf = NULL;
-	      size_t n;
-	      rewind (env->file);
-	      while (getline (&buf, &n, env->file) > 0)
-		fputs (buf, fp);
-	      fclose (fp);
-	      free (buf);
-	    }
-	}
-
-      fclose (env->file);
-      remove (filename);
-      free (filename);
+      save_dead_message (env);
+      mu_stream_destroy (&env->compstr);
       return 1;
     }
 
-  fclose (env->file);		/* FIXME: freopen would be better */
-
   /* In mailx compatibility mode, ask for Cc and Bcc after editing
      the body of the message */
   if (mailvar_get (NULL, "mailx", mailvar_type_boolean, 0) == 0)
@@ -515,123 +559,100 @@ mail_send0 (compose_env_t * env, int save_to)
 
   if (util_header_expand (&env->header) == 0)
     {
-      file = fopen (filename, "r");
-      if (file != NULL)
+      mu_message_t msg = NULL;
+      int rc;
+      int status = 0;
+      
+      mu_message_create (&msg, NULL);
+      
+      /* Fill the body.  */
+      mu_stream_seek (env->compstr, 0, MU_SEEK_SET, NULL);
+      rc = fill_body (msg, env->compstr);
+
+      if (rc == 0)
 	{
-	  mu_mailer_t mailer;
-	  mu_message_t msg = NULL;
-	  int rc;
-	  
-	  mu_message_create (&msg, NULL);
-	  mu_message_set_header (msg, env->header, NULL);
-
-	  /* Fill the body.  */
-	  rc = fill_body (msg, file);
-	  fclose (file);
-
-	  if (rc == 0)
+	  /* Save outgoing message */
+	  if (save_to)
 	    {
-	      /* Save outgoing message */
-	      if (save_to)
-		{
-		  char *tmp = compose_header_get (env, MU_HEADER_TO, NULL);
-		  mu_address_t addr = NULL;
+	      char *tmp = compose_header_get (env, MU_HEADER_TO, NULL);
+	      mu_address_t addr = NULL;
 	      
-		  mu_address_create (&addr, tmp);
-		  mu_address_aget_email (addr, 1, &savefile);
-		  mu_address_destroy (&addr);
-		  if (savefile)
-		    {
-		      char *p = strchr (savefile, '@');
-		      if (p)
-			*p = 0;
-		    }
-		}
-	      util_save_outgoing (msg, savefile);
+	      mu_address_create (&addr, tmp);
+	      mu_address_aget_email (addr, 1, &savefile);
+	      mu_address_destroy (&addr);
 	      if (savefile)
-		free (savefile);
-
-	      /* Check if we need to save the message to files or pipes.  */
-	      if (env->outfiles)
 		{
-		  int i;
-		  for (i = 0; i < env->nfiles; i++)
-		    {
-		      /* Pipe to a cmd.  */
-		      if (env->outfiles[i][0] == '|')
-			msg_to_pipe (&(env->outfiles[i][1]), msg);
-		      /* Save to a file.  */
-		      else
-			{
-			  int status;
-			  mu_mailbox_t mbx = NULL;
-			  status = mu_mailbox_create_default (&mbx, 
-                                                              env->outfiles[i]);
-			  if (status == 0)
-			    {
-			      status = mu_mailbox_open (mbx, MU_STREAM_WRITE
-							| MU_STREAM_CREAT);
-			      if (status == 0)
-				{
-				  status = mu_mailbox_append_message (mbx, msg);
-				  if (status)
-				    util_error (_("Cannot append message: %s"),
-						mu_strerror (status));
-				  mu_mailbox_close (mbx);
-				}
-			      mu_mailbox_destroy (&mbx);
-			    }
-			  if (status)
-			    util_error (_("Cannot create mailbox %s: %s"), 
-					env->outfiles[i], mu_strerror (status));
-			}
-		    }
-		}
-
-	      /* Do we need to Send the message on the wire?  */
-	      if (compose_header_get (env, MU_HEADER_TO, NULL)
-		  || compose_header_get (env, MU_HEADER_CC, NULL)
-		  || compose_header_get (env, MU_HEADER_BCC, NULL))
-		{
-		  char *sendmail;
-		  if (mailvar_get (&sendmail, "sendmail", mailvar_type_string, 0) 
-                       == 0)
-		    {
-		      int status = mu_mailer_create (&mailer, sendmail);
-		      if (status == 0)
-			{
-			  if (mailvar_get (NULL, "verbose", mailvar_type_boolean, 0)
-			      == 0)
-			    {
-			      mu_debug_t debug = NULL;
-			      mu_mailer_get_debug (mailer, &debug);
-			      mu_debug_set_level (debug,
-				         MU_DEBUG_LEVEL_UPTO (MU_DEBUG_PROT));
-			    }
-			  status = mu_mailer_open (mailer, MU_STREAM_RDWR);
-			  if (status == 0)
-			    {
-			      mu_mailer_send_message (mailer, msg, NULL, NULL);
-			      mu_mailer_close (mailer);
-			    }
-			  mu_mailer_destroy (&mailer);
-			}
-		      if (status != 0)
-			msg_to_pipe (sendmail, msg);
-		    }
-		  else
-		    util_error (_("Variable sendmail not set: no mailer"));
+		  char *p = strchr (savefile, '@');
+		  if (p)
+		    *p = 0;
 		}
 	    }
-	  mu_message_destroy (&msg, NULL);
-	  remove (filename);
-	  free (filename);
-	  return 0;
-	}
-    }
+	  util_save_outgoing (msg, savefile);
+	  if (savefile)
+	    free (savefile);
 
-  remove (filename);
-  free (filename);
+	  /* Check if we need to save the message to files or pipes.  */
+	  if (env->outfiles)
+	    {
+	      int i;
+	      for (i = 0; i < env->nfiles; i++)
+		{
+		  /* Pipe to a cmd.  */
+		  if (env->outfiles[i][0] == '|')
+		    status = msg_to_pipe (env->outfiles[i] + 1, msg);
+		  /* Save to a file.  */
+		  else
+		    {
+		      mu_mailbox_t mbx = NULL;
+		      status = mu_mailbox_create_default (&mbx, 
+							  env->outfiles[i]);
+		      if (status == 0)
+			{
+			  status = mu_mailbox_open (mbx, MU_STREAM_WRITE
+						    | MU_STREAM_CREAT);
+			  if (status == 0)
+			    {
+			      status = mu_mailbox_append_message (mbx, msg);
+			      if (status)
+				mu_error (_("Cannot append message: %s"),
+					    mu_strerror (status));
+			      mu_mailbox_close (mbx);
+			    }
+			  mu_mailbox_destroy (&mbx);
+			}
+		      if (status)
+			mu_error (_("Cannot create mailbox %s: %s"), 
+				    env->outfiles[i],
+				    mu_strerror (status));
+		    }
+		}
+	    }
+	  
+	  /* Do we need to Send the message on the wire?  */
+	  if (status == 0 &&
+	      (compose_header_get (env, MU_HEADER_TO, NULL) ||
+	       compose_header_get (env, MU_HEADER_CC, NULL) ||
+	       compose_header_get (env, MU_HEADER_BCC, NULL)))
+	    {
+	      mu_message_set_header (msg, env->header, NULL);
+	      env->header = NULL;
+	      status = send_message (msg);
+	      if (status)
+		{
+		  mu_error (_("cannot send message: %s"),
+			    mu_strerror (status));
+		  save_dead_message (env);
+		}
+	    }
+	}  
+      mu_stream_destroy (&env->compstr);
+      mu_message_destroy (&msg, NULL);
+      return status;
+    }
+  else
+    save_dead_message (env);
+  
+  mu_stream_destroy (&env->compstr);
   return 1;
 }
 
@@ -648,26 +669,33 @@ isfilename (const char *p)
 
 /* FIXME: Should probably be in util.c.  */
 /* Call popen(cmd) and write the message to it.  */
-static void
+static int
 msg_to_pipe (const char *cmd, mu_message_t msg)
 {
-  FILE *fp = popen (cmd, "w");
-  if (fp)
+  mu_stream_t progstream, msgstream;
+  int status, rc;
+  
+  status = mu_command_stream_create (&progstream, cmd, MU_STREAM_WRITE);
+  if (status)
     {
-      mu_stream_t stream = NULL;
-      char buffer[512];
-      off_t off = 0;
-      size_t n = 0;
-      mu_message_get_stream (msg, &stream);
-      while (mu_stream_read (stream, buffer, sizeof buffer - 1, off, &n) == 0
-	     && n != 0)
-	{
-	  buffer[n] = '\0';
-	  fprintf (fp, "%s", buffer);
-	  off += n;
-	}
-      pclose (fp);
+      mu_error (_("Cannot pipe to %s: %s"), cmd, mu_strerror (status));
+      return status;
     }
-  else
-    util_error (_("Piping %s failed"), cmd);
+
+  mu_message_get_streamref (msg, &msgstream);
+  status = mu_stream_copy (progstream, msgstream, 0, NULL);
+  rc = mu_stream_close (progstream);
+
+  if (status == 0 && rc)
+    status = rc;
+
+  mu_stream_destroy (&progstream);
+  mu_stream_destroy (&msgstream);
+  
+  if (status)
+    {
+      mu_error (_("Sending data to %s failed: %s"), cmd,
+		  mu_strerror (status));
+    }
+  return status;
 }
