@@ -1,5 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001, 2007 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2001, 2007, 2010, 2011 Free Software Foundation,
+   Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,9 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include "mail.h"
 
@@ -29,9 +28,11 @@
 int
 mail_set (int argc, char **argv)
 {
+  int flags = MOPTF_OVERWRITE | ((strcmp (argv[0], "setq") == 0) ? MOPTF_QUIET : 0);
+
   if (argc < 2)
     {
-      util_printenv (1);
+      mailvar_print (1);
       return 0;
     }
   else
@@ -40,30 +41,30 @@ mail_set (int argc, char **argv)
 
       for (i = 1; i < argc; i++)
 	{
-	  if (!strncmp ("no", argv[i], 2))
+	  char *value = strchr (argv[i], '=');
+	  if (value)
+	    *value++ = 0;
+	  
+	  if (!strncmp ("no", argv[i], 2) && !value)
 	    {
-	      util_setenv (&argv[i][2], NULL, Mail_env_boolean, 1);
+	      mailvar_set (&argv[i][2], NULL, mailvar_type_boolean,
+			   flags | MOPTF_UNSET);
 	    }
-	  else if (i+1 < argc && argv[i+1][0] == '=')
+	  else if (value)
 	    {
 	      int nval;
-	      char *name = argv[i];
 	      char *p;
 	      
-	      i += 2;
-	      if (i >= argc)
-		break;
-
-	      nval = strtoul (argv[i], &p, 0);
+	      nval = strtoul (value, &p, 0);
 	      if (*p == 0)
-		util_setenv (name, &nval, Mail_env_number, 1);
+		mailvar_set (argv[i], &nval, mailvar_type_number, flags);
 	      else
-		util_setenv (name, argv[i], Mail_env_string, 1);
+		mailvar_set (argv[i], value, mailvar_type_string, flags);
 	    }
 	  else
 	    {
 	      int dummy = 1;
-	      util_setenv (argv[i], &dummy, Mail_env_boolean, 1);
+	      mailvar_set (argv[i], &dummy, mailvar_type_boolean, flags);
 	    }
 	}
       return 0;
