@@ -1,5 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001, 2002, 2007 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2001, 2002, 2007, 2010, 2011 Free Software
+   Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,9 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include "mail.h"
 
@@ -28,7 +27,7 @@ mail_unalias (int argc, char **argv)
   if (argc == 1)
     {
       /* TRANSLATORS: 'unalias' is a command name. Do not translate it! */
-      util_error (_("unalias requires at least one argument"));
+      mu_error (_("unalias requires at least one argument"));
       return 1;
     }
   while (--argc)
