@@ -1,5 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001, 2007 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2001, 2007, 2010, 2011 Free Software Foundation,
+   Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,9 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include "mail.h"
 
@@ -33,12 +32,12 @@ mail_unset (int argc, char **argv)
   else
     {
       int status = 0, i = 1;
-      for (i=1; i < argc; i++)
+      for (i = 1; i < argc; i++)
 	{
 	  char *buf = xmalloc ((7+strlen (argv[i])) * sizeof (char));
 	  strcpy (buf, "set no");
 	  strcat (buf, argv[i]);
-	  if (!util_do_command (buf))
+	  if (!util_do_command ("%s", buf))
 	    status = 1;
 	  free (buf);
 	}
