@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001, 2002, 2003, 2005, 
-   2007, 2009 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2001, 2002, 2003, 2005, 2007, 2009, 2010, 2011
+   Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,9 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include "mail.h"
 
@@ -28,10 +26,8 @@
 int
 mail_write (int argc, char **argv)
 {
-  mu_message_t msg;
-  mu_body_t bod;
-  mu_stream_t stream;
-  FILE *output;
+  int rc;
+  mu_stream_t output;
   char *filename = NULL;
   msgset_t *msglist = NULL, *mp;
   int sender = 0;
@@ -47,10 +43,10 @@ mail_write (int argc, char **argv)
       char *p = NULL;
       if (n == 0)
         {
-          util_error (_("No applicable message"));
+          mu_error (_("No applicable message"));
           return 1;
         }
-      asprintf (&p, "%u", n);
+      mu_asprintf (&p, "%lu", (unsigned long) n);
       filename = util_outfolder_name (p);
       free (p);
     }
@@ -58,7 +54,7 @@ mail_write (int argc, char **argv)
   if (msgset_parse (argc, argv, MSG_NODELETED|MSG_SILENT, &msglist))
     {
       if (filename)
-          free (filename);
+	free (filename);
       return 1;
     }
 
@@ -72,53 +68,56 @@ mail_write (int argc, char **argv)
 	}
     }
 
-  output = fopen (filename, "a");
-  if (!output)
+  rc = mu_file_stream_create (&output, filename,
+			      MU_STREAM_APPEND|MU_STREAM_CREAT);
+  if (rc)
     {
-      util_error (_("can't open %s: %s"), filename, strerror (errno));
+      mu_error (_("can't open %s: %s"), filename, mu_strerror (rc));
       free (filename);
-      fclose (output);
       msgset_free (msglist);
       return 1;
     }
 
   for (mp = msglist; mp; mp = mp->next)
     {
+      mu_message_t msg;
+      mu_body_t body;
+      mu_stream_t stream;
       mu_attribute_t attr;
-      char buffer[512];
-      off_t off = 0;
-      size_t n = 0;
 
       if (util_get_message (mbox, mp->msg_part[0], &msg))
         continue;
 
-      mu_message_get_body (msg, &bod);
+      mu_message_get_body (msg, &body);
 
-      mu_body_size (bod, &size);
-      total_size += size;
-      mu_body_lines (bod, &size);
-      total_lines += size;
+      mu_body_get_streamref (body, &stream);
+      rc = mu_stream_copy (output, stream, 0, NULL);
+      mu_stream_destroy (&stream);
 
-      mu_body_get_stream (bod, &stream);
-      /* should there be a separator? */
-      while (mu_stream_read(stream, buffer, sizeof (buffer) - 1, off, &n) == 0
-	     && n != 0)
+      if (rc == 0)
 	{
-	  buffer[n] = '\0';
-	  fprintf (output, "%s", buffer);
-	  off += n;
+	  mu_body_size (body, &size);
+	  total_size += size;
+	  mu_body_lines (body, &size);
+	  total_lines += size;
+	  
+	  /* mark as saved. */
+	  
+	  mu_message_get_attribute (msg, &attr);
+	  mu_attribute_set_userflag (attr, MAIL_ATTRIBUTE_SAVED);
 	}
-
-      /* mark as saved. */
-
-      mu_message_get_attribute (msg, &attr);
-      mu_attribute_set_userflag (attr, MAIL_ATTRIBUTE_SAVED);
+      else
+	mu_error (_("cannot save %lu: %s"),
+		  (unsigned long) mp->msg_part[0], mu_strerror (rc));
     }
 
-  fprintf (ofile, "\"%s\" %3d/%-5d\n", filename, total_lines, total_size);
+  mu_stream_close (output);
+  mu_stream_destroy (&output);
+  
+  mu_printf ("\"%s\" %3lu/%-5lu\n", filename,
+		    (unsigned long) total_lines, (unsigned long) total_size);
 
   free (filename);
-  fclose (output);
   msgset_free (msglist);
   return 0;
 }
