@@ -1,9 +1,10 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001, 2002 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2001, 2002, 2005, 2007, 2010, 2011 Free Software
+   Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    GNU Mailutils is distributed in the hope that it will be useful,
@@ -12,9 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include "mail.h"
 
@@ -33,7 +32,7 @@
 
 static int
 z_parse_args(int argc, char **argv,
-	     unsigned int *return_count, int *return_dir)
+	     off_t *return_count, int *return_dir)
 {
   int count = 1;
   int mul = 1;
@@ -62,7 +61,7 @@ z_parse_args(int argc, char **argv,
 	 dir = D_NONE;
 	 break;
        default:
-	 util_error (_("Bad arguments for the scrolling command"));
+	 mu_error (_("Bad arguments for the scrolling command"));
 	 return 1;
        }
 
@@ -77,7 +76,7 @@ z_parse_args(int argc, char **argv,
 
       if (argc > 1)
 	{
-	  util_error (_("Too many arguments for the scrolling command"));
+	  mu_error (_("Too many arguments for the scrolling command"));
 	  return 1;
 	}
 
@@ -85,13 +84,13 @@ z_parse_args(int argc, char **argv,
 	{
 	  if (dir == D_NONE)
 	    {
-	      util_error (_("Argument not applicable for z"));
+	      mu_error (_("Argument not applicable for z"));
 	      return 1;
 	    }
 
 	  if ((mul = strtoul (argp, NULL, 10)) == 0)
 	    {
-	      util_error (_("Bad number of pages"));
+	      mu_error (_("Bad number of pages"));
 	      return 1;
 	    }
 	}
@@ -107,48 +106,29 @@ z_parse_args(int argc, char **argv,
 int
 mail_z (int argc, char **argv)
 {
-  unsigned int i, nlines;
-  unsigned int pagelines = util_screen_lines();
-  unsigned int count;
+  off_t count;
   int dir;
-  int crs, end;
-  
+  unsigned int pagelines = util_screen_lines ();
+
   if (z_parse_args(argc, argv, &count, &dir))
     return 1;
 
-  nlines = pagelines;
-
   count *= pagelines;
-  crs = cursor;
+
   switch (dir)
     {
     case D_BWD:
-      if (crs < nlines)
+      if (page_move (-count) == 0)
 	{
-	  fprintf (stdout, _("On first screenful of messages\n"));
+	  mu_printf (_("On first screenful of messages\n"));
 	  return 0;
 	}
-      if (crs < count)
-	crs = 1;
-      else
-	crs -= count;
       break;
 
     case D_FWD:
-      if (crs + pagelines > total)
+      if (page_move (count) == 0)
 	{
-	  fprintf (stdout, _("On last screenful of messages\n"));
-	  return 0;
-	}
-
-      crs += count;
-
-      if (crs + nlines > total)
-	nlines = total - crs + 1;
-
-      if (nlines <= 0)
-	{
-	  fprintf (stdout, _("On last screenful of messages\n"));
+	  mu_printf (_("On last screenful of messages\n"));
 	  return 0;
 	}
       break;
@@ -160,43 +140,13 @@ mail_z (int argc, char **argv)
 	   of the last message.  This behaviour is used on startup
 	   when displaying the summary and the headers, new messages
 	   are last but we want to display a screenful with the
-	   real crs set by summary() to the new message.  */
-
-	/* Find the start of the last screen page.  */
-	int lastpage =  total - pagelines + 1;
-	if (lastpage <= 0)
-	  lastpage = 1;
-
-	if (crs > (unsigned int)lastpage)
-	  {
-	    crs = lastpage;
-
-	    if (crs + nlines > total)
-	      nlines = total - crs;
-
-	    util_range_msg (crs, crs + nlines,
-			    MSG_NODELETED|MSG_SILENT, mail_from0, NULL);
-	    return 1;
-	  }
-	else if (crs + nlines > total)
-	  nlines = total - crs + 1;
-      }
-      break;
-    }
-
-  cursor = crs;
-  end = cursor + nlines - 1;
-  i = 0;
-  do
-    {
-      int cnt = util_range_msg (crs, end,
-				MSG_NODELETED|MSG_SILENT, mail_from0, NULL);
-      if (cnt == 0)
+	   real crs set by summary() to the new message.
+	   FIXME: Basically it's the same as headers now. Do we need
+	   it still? */
 	break;
-      i += cnt;
-      crs += nlines;
+      }
     }
-  while (i < nlines && crs <= total);
 
-  return 1;
+  page_do (mail_from0, NULL);
+  return 0;
 }
