@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2005,
-   2006, 2007, 2008  Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2005, 2006, 2007, 2008,
+   2009, 2010, 2011 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,40 +13,34 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 /* fmtcheck */
 
 #include <mh.h>
 
-const char *program_version = "fmtcheck (" PACKAGE_STRING ")";
 static char doc[] = N_("GNU MH fmtcheck")"\v"
 N_("Use -help to obtain the list of traditional MH options.");
-static char args_doc[] = "";
 
 /* GNU options */
 static struct argp_option options[] = {
   {"form",    ARG_FORM, N_("FILE"),   0,
-   N_("Read format from given file")},
+   N_("read format from given file")},
   {"format",  ARG_FORMAT, N_("FORMAT"), 0,
-   N_("Use this format string")},
+   N_("use this format string")},
   {"dump",    ARG_DUMP, NULL,     0,
-   N_("Dump the listing of compiled format code")},
+   N_("dump the listing of compiled format code")},
   { "debug",  ARG_DEBUG, NULL,     0,
-    N_("Enable parser debugging output"),},
-  {"license", ARG_LICENSE, 0,      0,
-   N_("Display software license"), -1},
+    N_("enable parser debugging output"),},
 
-  { 0 }
+  { NULL }
 };
 
 /* Traditional MH options */
 struct mh_option mh_option[] = {
-  {"form",    4,  MH_OPT_ARG, "formatfile"},
-  {"format",  5,  MH_OPT_ARG, "string"},
-  { 0 }
+  { "form",    MH_OPT_ARG, "formatfile" },
+  { "format",  MH_OPT_ARG, "string" },
+  { NULL }
 };
 
 char *format_str;
@@ -68,8 +62,8 @@ action_dump ()
 
 static action_fp action = action_dump;
 
-static int
-opt_handler (int key, char *arg, void *unused, struct argp_state *state)
+static error_t
+opt_handler (int key, char *arg, struct argp_state *state)
 {
   switch (key)
     {
@@ -89,12 +83,8 @@ opt_handler (int key, char *arg, void *unused, struct argp_state *state)
       mh_format_debug (1);
       break;
       
-    case ARG_LICENSE:
-      mh_license (argp_program_version);
-      break;
-
     default:
-      return 1;
+      return ARGP_ERR_UNKNOWN;
     }
   return 0;
 }
@@ -105,8 +95,8 @@ main (int argc, char **argv)
   /* Native Language Support */
   MU_APP_INIT_NLS ();
 
-  mh_argp_init (program_version);
-  mh_argp_parse (&argc, &argv, 0, options, mh_option, args_doc, doc,
+  mh_argp_init ();
+  mh_argp_parse (&argc, &argv, 0, options, mh_option, NULL, doc,
 		 opt_handler, NULL, NULL);
 
   if (format_str && mh_format_parse (format_str, &format))
