@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
    Copyright (C) 1999, 2000, 2001, 2002, 2004, 2005, 2006, 2007, 2009,
-   2010 Free Software Foundation, Inc.
+   2010, 2011 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,9 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -27,7 +25,7 @@
 #include <unistd.h>
 #include <mh_getopt.h>
 #include <xalloc.h>
-#include <mu_asprintf.h>
+#include <mailutils/io.h>
 
 #include <string.h>
 
@@ -42,7 +40,7 @@
 #include <mailutils/list.h>
 #include <mailutils/iterator.h>
 #include <mailutils/address.h>
-#include <mailutils/mutil.h>
+#include <mailutils/util.h>
 #include <mailutils/stream.h>
 #include <mailutils/filter.h>
 #include <mailutils/url.h>
@@ -51,18 +49,19 @@
 #include <mailutils/errno.h>
 #include <mailutils/nls.h>
 #include <mailutils/argcv.h>
+#include <mailutils/wordsplit.h>
 #include <mailutils/debug.h>
 #include <mailutils/mailer.h>
 #include <mailutils/envelope.h>
 #include <mailutils/mime.h>
+#include <mailutils/io.h>
+#include <mailutils/property.h>
+#include <mailutils/prog.h>
+#include <mailutils/mh.h>
+#include <mailutils/stdstream.h>
 
-#include <mu_asprintf.h>
 #include <mu_umaxtostr.h>
 
-#if !HAVE_DECL_STRCHRNUL
-extern char *strchrnul (const char *s, int c_in);
-#endif
-
 #define MH_FMT_RALIGN  0x1000
 #define MH_FMT_ZEROPAD 0x2000
 #define MH_FMT_COMPWS  0x4000
@@ -194,10 +193,14 @@ typedef struct
   mu_header_t header;
 } mh_context_t;
 
+#define MH_MSGSET_UID   0x01
+
 typedef struct
 {
-  size_t count;
+  int flags;
   size_t *list;
+  size_t count;
+  size_t size;
 } mh_msgset_t;
 
 typedef void (*mh_iterator_fp) (mu_mailbox_t mbox, mu_message_t msg,
@@ -217,26 +220,23 @@ struct mh_whatnow_env     /* whatnow shell environment */
   char *file;             /* The file being processed */
   char *msg;              /* File name of the original message (if any) */
   char *draftfile;        /* File to preserve the draft into */
-  const char *draftfolder;
   const char *editor;
   char *prompt;
   char *anno_field;       /* Annotate field to be used */
   mu_list_t anno_list;    /* List of messages (mu_message_t) to annotate */
+  mu_mailbox_t mbox;
 };
 
 #define DISP_QUIT 0
 #define DISP_USE 1
 #define DISP_REPLACE 2
 
-typedef int (*mh_context_iterator) (const char *field, const char *value,
-				    void *data);
-
 #define SEQ_PRIVATE 1
 #define SEQ_ZERO    2
 
-extern size_t current_message;
 extern char mh_list_format[];
 extern int rcpt_mask;
+extern int mh_mailbox_cur_default;
 
 void mh_init (void);
 void mh_init2 (void);
@@ -246,19 +246,36 @@ mu_message_t mh_file_to_message (const char *folder, const char *file_name);
 mu_message_t mh_stream_to_message (mu_stream_t stream);
 void mh_install (char *name, int automode);
 
-const char *mh_global_profile_get (const char *name, const char *defval);
-int mh_global_profile_set (const char *name, const char *value);
-const char *mh_global_context_get (const char *name, const char *defval);
-int mh_global_context_set (const char *name, const char *value);
+mu_property_t mh_read_property_file (char *name, int ro);
+void mh_property_merge (mu_property_t dst, mu_property_t src);
+
+#define mh_global_profile_get(name, defval) \
+  mu_mhprop_get_value (mu_mh_profile, name, defval)
+#define mh_global_profile_set(name, value) \
+  mu_property_set_value (mu_mh_profile, name, value, 1))
+#define mh_global_profile_iterate(fp, data) \
+  mu_mhprop_iterate (mu_mh_profile, fp, data)
+
+#define mh_global_context_get(name, defval) \
+  mu_mhprop_get_value (mu_mh_context, name, defval)
+#define mh_global_context_set(name, value) \
+  mu_property_set_value (mu_mh_context, name, value, 1)
+#define mh_global_context_iterate(fp, data) \
+  mu_mhprop_iterate (mu_mh_context, fp, data)
+
 const char *mh_set_current_folder (const char *val);
 const char *mh_current_folder (void);
-const char *mh_global_sequences_get (const char *name, const char *defval);
-int mh_global_sequences_set (const char *name, const char *value);
+
+mu_property_t mh_mailbox_get_property (mu_mailbox_t mbox);
+const char *mh_global_sequences_get (mu_mailbox_t mbox,
+				     const char *name, const char *defval);
+void mh_global_sequences_set (mu_mailbox_t mbox,
+			      const char *name, const char *value);
+void mh_global_sequences_iterate (mu_mailbox_t mbox,
+				  mu_mhprop_iterator_t fp, void *data);
+void mh_global_sequences_drop (mu_mailbox_t mbox);
+
 void mh_global_save_state (void);
-int mh_global_profile_iterate (mh_context_iterator fp, void *data);
-int mh_global_context_iterate (mh_context_iterator fp, void *data);
-int mh_global_sequences_iterate (mh_context_iterator fp, void *data);
-void mh_global_sequences_drop (void);
 
 int mh_interactive_mode_p (void);
 int mh_getyn (const char *fmt, ...) MU_PRINTFLIKE(1,2);
@@ -281,31 +298,21 @@ void mh_err_memory (int fatal);
 FILE *mh_audit_open (char *name, mu_mailbox_t mbox);
 void mh_audit_close (FILE *fp);
 
-mh_context_t *mh_context_create (const char *name, int copy);
-int mh_context_read (mh_context_t *ctx);
-int mh_context_write (mh_context_t *ctx);
-const char *mh_context_get_value (mh_context_t *ctx, const char *name,
-				  const char *defval);
-int mh_context_set_value (mh_context_t *ctx, const char *name,
-			  const char *value);
-int mh_context_iterate (mh_context_t *ctx, mh_context_iterator fp, void *data);
-void mh_context_destroy (mh_context_t **pctx);
-void mh_context_merge (mh_context_t *dst, mh_context_t *src);
-
 int mh_message_number (mu_message_t msg, size_t *pnum);
 
-mu_mailbox_t mh_open_folder (const char *folder, int create);
+mu_mailbox_t mh_open_folder (const char *folder, int flags);
 
-int mh_msgset_parse (mu_mailbox_t mbox, mh_msgset_t *msgset,
-		     int argc, char **argv, char *def);
+void mh_msgset_parse (mu_mailbox_t mbox, mh_msgset_t *msgset,
+		      int argc, char **argv, char *def);
 int mh_msgset_member (mh_msgset_t *msgset, size_t num);
 void mh_msgset_reverse (mh_msgset_t *msgset);
 void mh_msgset_negate (mu_mailbox_t mbox, mh_msgset_t *msgset);
-int mh_msgset_current (mu_mailbox_t mbox, mh_msgset_t *msgset, int index);
+void mh_msgset_current (mu_mailbox_t mbox, mh_msgset_t *msgset, int index);
 void mh_msgset_free (mh_msgset_t *msgset);
 void mh_msgset_uids (mu_mailbox_t mbox, mh_msgset_t *msgset);
 
 char *mh_get_dir (void);
+int mh_find_file (const char *name, char **resolved_name);
 char *mh_expand_name (const char *base, const char *name, int is_folder);
 void mh_quote (const char *in, char **out);
 void mh_expand_aliases (mu_message_t msg, mu_address_t *addr_to,
@@ -326,6 +333,9 @@ int mh_draft_message (const char *name, const char *msgspec, char **pname);
      
 int mh_spawnp (const char *prog, const char *file);
 int mh_whatnow (struct mh_whatnow_env *wh, int initial_edit);
+int mh_whatnowproc (struct mh_whatnow_env *wh, int initial_edit,
+		    const char *prog);
+
 int mh_disposition (const char *filename);
 int mh_usedraft (const char *filename);
 int mh_file_copy (const char *from, const char *to);
@@ -355,18 +365,27 @@ void mh_annotate (mu_message_t msg, char *field, char *text, int date);
 #define MHL_DISABLE_BODY 8
 
 mu_list_t mhl_format_compile (char *name);
+
 int mhl_format_run (mu_list_t fmt, int width, int length, int flags,
 		    mu_message_t msg, mu_stream_t output);
 void mhl_format_destroy (mu_list_t *fmt);
 
-void mh_seq_add (const char *name, mh_msgset_t *mset, int flags);
-int mh_seq_delete (const char *name, mh_msgset_t *mset, int flags);
-const char *mh_seq_read (const char *name, int flags);
+void mh_seq_add (mu_mailbox_t mbox, const char *name, mh_msgset_t *mset,
+		 int flags);
+int mh_seq_delete (mu_mailbox_t mbox, const char *name, mh_msgset_t *mset,
+		   int flags);
+const char *mh_seq_read (mu_mailbox_t mbox, const char *name, int flags);
 
-void mh_comp_draft (const char *formfile, const char *defformfile,
-		    const char *draftfile);
+void mh_comp_draft (const char *formfile, const char *draftfile);
 int check_draft_disposition (struct mh_whatnow_env *wh, int use_draft);
 
 void ali_parse_error (const char *fmt, ...) MU_PRINTFLIKE(1,2); 
 void ali_verbatim (int enable);
 
+char *mh_safe_make_file_name (const char *dir, const char *file);
+  
+void mh_mailbox_get_cur (mu_mailbox_t mbox, size_t *pcur);
+void mh_mailbox_set_cur (mu_mailbox_t mbox, size_t cur);
+
+void mh_whatnow_env_from_environ (struct mh_whatnow_env *wh);
+void mh_whatnow_env_to_environ (struct mh_whatnow_env *wh);
