@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2005, 2006, 2007, 2009,
-   2010 Free Software Foundation, Inc.
+   2010, 2011 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,9 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 /* This module implements execution of MH format strings. */
 
@@ -603,14 +601,16 @@ mh_format (mh_format_t *fmt, mu_message_t msg, size_t msgno,
 	  {
 	    mu_body_t body = NULL;
 	    mu_stream_t stream = NULL;
-	    size_t size = 0, off, str_off, nread;
+	    size_t size = 0, str_off, nread;
 	    size_t rest = DFLWIDTH (&mach);
 
 	    strobj_free (&mach.arg_str);
 	    mu_message_get_body (mach.message, &body);
 	    mu_body_size (body, &size);
-	    mu_body_get_stream (body, &stream);
-	    if (size == 0 || !stream)
+	    if (size == 0)
+	      break;
+	    mu_body_get_streamref (body, &stream);
+	    if (!stream)
 	      break;
 	    if (size > rest)
 	      size = rest;
@@ -618,18 +618,17 @@ mh_format (mh_format_t *fmt, mu_message_t msg, size_t msgno,
 	    mach.arg_str.ptr = xmalloc (size+1);
 	    mach.arg_str.size = size;
 	    
-	    off = 0;
 	    str_off = 0;
 	    while (!mu_stream_read (stream, mach.arg_str.ptr + str_off,
-				 mach.arg_str.size - str_off, off, &nread)
+				    mach.arg_str.size - str_off, &nread)
 		   && nread != 0
 		   && str_off < size)
 	      {
-		off += nread;
                 COMPRESS_WS (&mach, mach.arg_str.ptr + str_off, &nread);
 		if (nread)
 		  str_off += nread;
 	      }
+	    mu_stream_destroy (&stream);
 	    mach.arg_str.ptr[str_off] = 0;
 	  }
 	  break;
@@ -914,8 +913,19 @@ static void
 builtin_cur (struct mh_machine *mach)
 {
   size_t msgno = mach->msgno;
+  size_t cur;
+  int rc;
+  mu_mailbox_t mbox;
+  
+  rc = mu_message_get_mailbox (mach->message, &mbox);
+  if (rc)
+    {
+      mu_diag_funcall (MU_DIAG_ERROR, "mu_message_get_mailbox", NULL, rc);
+      exit (1);
+    }
   mh_message_number (mach->message, &msgno);
-  mach->arg_num = msgno == current_message;
+  mh_mailbox_get_cur (mbox, &cur); /* FIXME: Cache this */
+  mach->arg_num = msgno == cur;
 }
 
 static void
@@ -1137,7 +1147,7 @@ static void
 builtin_putnum (struct mh_machine *mach)
 {
   char *p;
-  asprintf (&p, "%ld", mach->arg_num);
+  mu_asprintf (&p, "%ld", mach->arg_num);
   print_string (mach, 0, p);
   free (p);
 }
@@ -1573,7 +1583,20 @@ builtin_proper (struct mh_machine *mach)
 static void
 builtin_friendly (struct mh_machine *mach)
 {
-  /*FIXME: noop*/
+  mu_address_t addr;
+  const char *str;
+  int rc;
+  
+  rc = mu_address_create (&addr, strobj_ptr (&mach->arg_str));
+  if (rc)
+    return;
+
+  if (mu_address_sget_personal (addr, 1, &str) == 0 && str)
+    {
+      strobj_free (&mach->arg_str);
+      strobj_create (&mach->arg_str, str);
+    }
+  mu_address_destroy (&addr);
 }
 
 /*     addr       addr     string   mbox@host or host!mbox rendering*/
@@ -1608,12 +1631,7 @@ builtin_pers (struct mh_machine *mach)
     return;
 
   if (mu_address_sget_personal (addr, 1, &str) == 0 && str)
-    {
-      char *p;
-      asprintf (&p, "\"%s\"", str);
-      strobj_create (&mach->arg_str, p);
-      free (p);
-    }
+    strobj_create (&mach->arg_str, str);
   mu_address_destroy (&addr);
 }
 
