@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002, 2003, 
-   2005, 2006, 2007, 2009 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2005, 2006, 2007, 2009,
+   2010, 2011 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,14 +13,13 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 /* Initialize MH applications. */
 
 #include <mh.h>
 #include <mailutils/url.h>
+#include <mailutils/tls.h>
 #include <pwd.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -44,6 +43,8 @@ char mh_list_format[] =
 void
 mh_init ()
 {
+  mu_stdstream_setup (MU_STDSTREAM_RESET_NONE);
+  
   /* Register all mailbox and mailer formats */
   mu_register_all_formats ();
 
@@ -55,7 +56,6 @@ void
 mh_init2 ()
 {
   mh_current_folder ();
-  mh_global_sequences_get ("cur", NULL);
 }
 
 int
@@ -66,20 +66,34 @@ mh_read_formfile (char *name, char **pformat)
   char *ptr;
   size_t off = 0;
   char *format_str;
-
-  if (stat (name, &st))
+  char *file_name;
+  int rc;
+  
+  rc = mh_find_file (name, &file_name);
+  if (rc)
     {
-      mu_error (_("cannot stat format file %s: %s"), name, strerror (errno));
+      mu_error (_("cannot access format file %s: %s"), name, strerror (rc));
       return -1;
     }
   
-  fp = fopen (name, "r");
-  if (!fp)
+  if (stat (file_name, &st))
     {
-      mu_error (_("cannot open format file %s: %s"), name, strerror (errno));
+      mu_error (_("cannot stat format file %s: %s"), file_name,
+		strerror (errno));
+      free (file_name);
       return -1;
     }
-
+  
+  fp = fopen (file_name, "r");
+  if (!fp)
+    {
+      mu_error (_("cannot open format file %s: %s"), file_name,
+		strerror (errno));
+      free (file_name);
+      return -1;
+    }
+  free (file_name);
+  
   format_str = xmalloc (st.st_size+1);
   while ((ptr = fgets (format_str + off, st.st_size - off + 1, fp)) != NULL)
     {
@@ -379,14 +393,7 @@ mh_audit_open (char *name, mu_mailbox_t mbox)
   namep = mu_tilde_expansion (name, "/", NULL);
   if (strchr (namep, '/') == NULL)
     {
-      char *p = NULL;
-
-      asprintf (&p, "%s/%s", mu_folder_directory (), namep);
-      if (!p)
-	{
-	  mu_error (_("not enough memory"));
-	  exit (1);
-	}
+      char *p = mh_safe_make_file_name (mu_folder_directory (), namep);
       free (namep);
       namep = p;
     }
@@ -425,14 +432,13 @@ mh_message_number (mu_message_t msg, size_t *pnum)
 }
 
 mu_mailbox_t
-mh_open_folder (const char *folder, int create)
+mh_open_folder (const char *folder, int flags)
 {
   mu_mailbox_t mbox = NULL;
   char *name;
-  int flags = MU_STREAM_RDWR;
   
   name = mh_expand_name (NULL, folder, 1);
-  if (create && mh_check_folder (name, 1))
+  if ((flags & MU_STREAM_CREAT) && mh_check_folder (name, 1))
     exit (0);
     
   if (mu_mailbox_create_default (&mbox, name))
@@ -442,9 +448,6 @@ mh_open_folder (const char *folder, int create)
       exit (1);
     }
 
-  if (create)
-    flags |= MU_STREAM_CREAT;
-  
   if (mu_mailbox_open (mbox, flags))
     {
       mu_error (_("cannot open mailbox %s: %s"), name, strerror (errno));
@@ -465,11 +468,16 @@ mh_get_dir ()
   if (mhdir[0] != '/')
     {
       char *p = mu_get_homedir ();
-      asprintf (&mhcopy, "%s/%s", p, mhdir);
+      mhcopy = mh_safe_make_file_name (p, mhdir);
       free (p);
     }
   else 
     mhcopy = strdup (mhdir);
+  if (!mhcopy)
+    {
+      mu_error (_("not enough memory"));
+      abort ();
+    }
   return mhcopy;
 }
 
@@ -485,8 +493,7 @@ mh_expand_name (const char *base, const char *name, int is_folder)
   else if (strncmp (namep, "../", 3) == 0 || strncmp (namep, "./", 2) == 0)
     {
       char *cwd = mu_getcwd ();
-      char *tmp = NULL;
-      asprintf (&tmp, "%s/%s", cwd, namep);
+      char *tmp = mh_safe_make_file_name (cwd, namep);
       free (cwd);
       free (namep);
       namep = tmp;
@@ -497,12 +504,13 @@ mh_expand_name (const char *base, const char *name, int is_folder)
       if (memcmp (namep, "mh:/", 4) == 0)
 	return namep;
       else if (namep[0] == '/')
-	asprintf (&p, "mh:%s", namep);
+	mu_asprintf (&p, "mh:%s", namep);
       else
-	asprintf (&p, "mh:%s/%s", base ? base : mu_folder_directory (), namep);
+	mu_asprintf (&p, "mh:%s/%s", base ? base : mu_folder_directory (), 
+                     namep);
     }
   else if (namep[0] != '/')
-    asprintf (&p, "%s/%s", base ? base : mu_folder_directory (), namep);
+    mu_asprintf (&p, "%s/%s", base ? base : mu_folder_directory (), namep);
   else
     return namep;
   
@@ -510,6 +518,64 @@ mh_expand_name (const char *base, const char *name, int is_folder)
   return p;
 }
 
+int
+mh_find_file (const char *name, char **resolved_name)
+{
+  char *s;
+  int rc;
+  
+  if (name[0] == '/' ||
+      (name[0] == '.' && name[1] == '/') ||
+      (name[0] == '.' && name[1] == '.' && name[2] == '/'))
+    {
+      *resolved_name = xstrdup (name);
+      if (access (name, R_OK) == 0)
+	return 0;
+      return errno;
+    }
+
+  if (name[0] == '~')
+    {
+      s = mu_tilde_expansion (name, "/", NULL);
+      *resolved_name = s;
+      if (access (s, R_OK) == 0)
+	return 0;
+      return errno;
+    }
+  
+  s = mh_expand_name (NULL, name, 0);
+  if (access (s, R_OK) == 0)
+    {
+      *resolved_name = s;
+      return 0;
+    }
+  if (errno != ENOENT)
+    mu_diag_output (MU_DIAG_WARNING,
+		    _("cannot access %s: %s"), s, mu_strerror (errno));
+  free (s);
+
+  s = mh_expand_name (mh_global_profile_get ("mhetcdir", MHLIBDIR), name, 0);
+  if (access (s, R_OK) == 0)
+    {
+      *resolved_name = s;
+      return 0;
+    }
+  if (errno != ENOENT)
+    mu_diag_output (MU_DIAG_WARNING,
+		    _("cannot access %s: %s"), s, mu_strerror (errno));
+  free (s);
+
+  *resolved_name = xstrdup (name);
+  if (access (name, R_OK) == 0)
+    return 0;
+  rc = errno;
+  if (rc != ENOENT)
+    mu_diag_output (MU_DIAG_WARNING,
+		    _("cannot access %s: %s"), s, mu_strerror (rc));
+
+  return rc;
+}
+
 int
 mh_iterate (mu_mailbox_t mbox, mh_msgset_t *msgset,
 	    mh_iterator_fp itr, void *data)
@@ -525,7 +591,8 @@ mh_iterate (mu_mailbox_t mbox, mh_msgset_t *msgset,
       num = msgset->list[i];
       if ((rc = mu_mailbox_get_message (mbox, num, &msg)) != 0)
 	{
-	  mu_error (_("cannot get message %d: %s"), num, mu_strerror (rc));
+	  mu_error (_("cannot get message %lu: %s"),
+		    (unsigned long) num, mu_strerror (rc));
 	  return 1;
 	}
 
@@ -538,99 +605,82 @@ mh_iterate (mu_mailbox_t mbox, mh_msgset_t *msgset,
 int
 mh_spawnp (const char *prog, const char *file)
 {
-  int argc, i, rc, status;
-  char **argv, **xargv;
+  struct mu_wordsplit ws;
+  size_t i;
+  int rc, status;
+  char **xargv;
 
-  if ((rc = mu_argcv_get (prog, "", "#", &argc, &argv)) != 0)
+  ws.ws_comment = "#";
+  if (mu_wordsplit (prog, &ws, MU_WRDSF_DEFFLAGS | MU_WRDSF_COMMENT))
     {
-      mu_diag_funcall (MU_DIAG_ERROR, "mu_argcv_get", prog, rc);
-      mu_argcv_free (argc, argv);
+      mu_error (_("cannot split line `%s': %s"), prog,
+		mu_wordsplit_strerror (&ws));
       return 1;
     }
 
-  xargv = calloc (argc + 2, sizeof (*xargv));
+  xargv = calloc (ws.ws_wordc + 2, sizeof (*xargv));
   if (!xargv)
     {
       mh_err_memory (0);
-      mu_argcv_free (argc, argv);
+      mu_wordsplit_free (&ws);
       return 1;
     }
 
-  for (i = 0; i < argc; i++)
-    xargv[i] = argv[i];
+  for (i = 0; i < ws.ws_wordc; i++)
+    xargv[i] = ws.ws_wordv[i];
   xargv[i++] = (char*) file;
   xargv[i++] = NULL;
-
+  
   rc = mu_spawnvp (xargv[0], xargv, &status);
 
   free (xargv);
-  mu_argcv_free (argc, argv);
+  mu_wordsplit_free (&ws);
 
   return rc;
 }
 
+/* Copy data from FROM to TO, creating the latter if necessary.
+   FIXME: How about formats?
+*/
 int
 mh_file_copy (const char *from, const char *to)
 {
-  char *buffer;
-  size_t bufsize, rdsize;
-  struct stat st;
-  mu_stream_t in;
-  mu_stream_t out;
+  mu_stream_t in, out, flt;
   int rc;
   
-  if (stat (from, &st))
-    {
-      mu_error ("mh_copy: %s", mu_strerror (errno));
-      return -1;
-    }
-
-  for (bufsize = st.st_size; bufsize > 0 && (buffer = malloc (bufsize)) == 0;
-       bufsize /= 2)
-    ;
-
-  if (!bufsize)
-    mh_err_memory (1);
-
-  if ((rc = mu_file_stream_create (&in, from, MU_STREAM_READ)) != 0
-      || (rc = mu_stream_open (in)))
+  if ((rc = mu_file_stream_create (&in, from, MU_STREAM_READ)))
     {
       mu_error (_("cannot open input file `%s': %s"),
 		from, mu_strerror (rc));
-      free (buffer);
       return 1;
     }
 
-  if ((rc = mu_file_stream_create (&out, to, MU_STREAM_RDWR|MU_STREAM_CREAT)) != 0
-      || (rc = mu_stream_open (out)))
+  if ((rc = mu_file_stream_create (&out, to, MU_STREAM_RDWR|MU_STREAM_CREAT)))
     {
       mu_error (_("cannot open output file `%s': %s"),
 		to, mu_strerror (rc));
-      free (buffer);
-      mu_stream_close (in);
-      mu_stream_destroy (&in, mu_stream_get_owner (in));
+      mu_stream_destroy (&in);
       return 1;
     }
 
-  while (st.st_size > 0
-	 && (rc = mu_stream_sequential_read (in, buffer, bufsize, &rdsize)) == 0
-	 && rdsize > 0)
+  rc = mu_filter_create (&flt, in, "INLINE-COMMENT", MU_FILTER_DECODE,
+			 MU_STREAM_READ);
+  mu_stream_unref (in);
+  if (rc)
     {
-      if ((rc = mu_stream_sequential_write (out, buffer, rdsize)) != 0)
-	{
-	  mu_error (_("write error on `%s': %s"),
-		    to, mu_strerror (rc));
-	  break;
-	}
-      st.st_size -= rdsize;
+      mu_error (_("cannot open filter stream: %s"), mu_strerror (rc));
+      mu_stream_destroy (&out);
+      return 1;
     }
+  
+  rc = mu_stream_copy (out, flt, 0, NULL);
+      
+  mu_stream_destroy (&flt);
+  mu_stream_destroy (&out);
 
-  free (buffer);
-
-  mu_stream_close (in);
-  mu_stream_close (out);
-  mu_stream_destroy (&in, mu_stream_get_owner (in));
-  mu_stream_destroy (&out, mu_stream_get_owner (out));
+  if (rc)
+    mu_error (_("error copying file `%s' to `%s': %s"),
+	      from, to, mu_strerror (rc));
   
   return rc;
 }
@@ -655,14 +705,6 @@ _file_to_message (const char *file_name)
       return NULL;
     }
   
-  if ((rc = mu_stream_open (instream)))
-    {
-      mu_error (_("cannot open input stream (file %s): %s"),
-		file_name, mu_strerror (rc));
-      mu_stream_destroy (&instream, mu_stream_get_owner (instream));
-      return NULL;
-    }
-
   return mh_stream_to_message (instream);
 }
 
@@ -703,14 +745,13 @@ mh_real_install (char *name, int automode)
   char *home = mu_get_homedir ();
   char *mhdir;
   char *ctx;
-  FILE *fp;
-  
-  asprintf (&mhdir, "%s/%s", home, "Mail");
+  int rc;
+  mu_stream_t profile;
+
+  mhdir = mh_safe_make_file_name (home, "Mail");
   
   if (!automode)
     {
-      size_t n = 0;
-      
       /* TRANSLATORS: This is a question and will be followed
 	 by question mark on output. */
       if (mh_getyn_interactive (_("Do you need help")))
@@ -721,29 +762,27 @@ mh_real_install (char *name, int automode)
       if (!mh_getyn_interactive (_("Do you want the standard MH path \"%s\""), mhdir))
 	{
 	  int local;
-	  char *p;
+	  char *p, *buf = NULL;
+	  size_t size = 0;
 	  
 	  /* TRANSLATORS: This is a question and will be followed
 	     by question mark on output. */
 	  local = mh_getyn_interactive (_("Do you want a path below your login directory"));
 	  if (local)
-	    printf (_("What is the path? "));
+	    mu_printf (_("What is the path? "));
 	  else
-	    printf (_("What is the full path? "));
-	  if (getline (&p, &n, stdin) <= 0)
+	    mu_printf (_("What is the full path? "));
+	  mu_stream_flush (mu_strin);
+	  if (mu_stream_getline (mu_strin, &buf, &size, NULL))
 	    exit (1);
-
-	  n = strlen (p);
-	  if (n == 0)
-	    exit (1);
-
-	  if (p[n-1] == '\n')
-	    p[n-1] = 0;
-
+	  p = mu_str_stripws (buf);
+	  if (p > buf)
+	    memmove (buf, p, strlen (p) + 1);
+	  
 	  free (mhdir);
 	  if (local)
 	    {
-	      asprintf (&mhdir, "%s/%s", home, p);
+              mhdir = mh_safe_make_file_name (home, p);
 	      free (p);
 	    }
 	  else
@@ -754,24 +793,26 @@ mh_real_install (char *name, int automode)
   if (mh_check_folder (mhdir, !automode))
     exit (1);
 
-  fp = fopen (name, "w");
-  if (!fp)
+  rc = mu_file_stream_create (&profile, name,
+			      MU_STREAM_WRITE | MU_STREAM_CREAT);
+  if (rc)
     {
-      mu_error (_("cannot open file %s: %s"), name, mu_strerror (errno));
+      mu_error (_("cannot open file %s: %s"), name, mu_strerror (rc));
       exit (1);
     }
-  fprintf (fp, "Path: %s\n", mhdir);
-  fclose (fp);
+  mu_stream_printf (profile, "Path: %s\n", mhdir);
+  mu_stream_destroy (&profile);
 
-  asprintf (&ctx, "%s/%s", mhdir, MH_CONTEXT_FILE);
-  fp = fopen (ctx, "w");
-  if (fp)
+  ctx = mh_safe_make_file_name (mhdir, MH_CONTEXT_FILE);
+  rc = mu_file_stream_create (&profile, ctx,
+			      MU_STREAM_WRITE | MU_STREAM_CREAT);
+  if (rc)
     {
-      fprintf (fp, "Current-Folder: inbox\n");
-      fclose (fp);
+      mu_stream_printf (profile, "Current-Folder: inbox\n");
+      mu_stream_destroy (&profile);
     }
   free (ctx);
-  asprintf (&ctx, "%s/inbox", mhdir);
+  ctx = mh_safe_make_file_name (mhdir, "inbox");
   if (mh_check_folder (ctx, !automode))
     exit (1);
   free (ctx);
@@ -783,7 +824,7 @@ mh_install (char *name, int automode)
 {
   struct stat st;
   
-  if (stat(name, &st))
+  if (stat (name, &st))
     {
       if (errno == ENOENT)
 	{
@@ -800,13 +841,12 @@ mh_install (char *name, int automode)
   else if ((st.st_mode & S_IFREG) || (st.st_mode & S_IFLNK)) 
     {
       mu_error(_("You already have an MH profile, use an editor to modify it"));
-      exit (0);
+      exit (1);
     }
   else
     {
-      mu_error(_("You already have file %s which is not a regular file or a symbolic link.\n"
-		 "Please remove it and try again"),
-	       name);
+      mu_error (_("You already have file %s which is not a regular file or a symbolic link."), name);
+      mu_error (_("Please remove it and try again"));
       exit (1);
     }
 }
@@ -865,36 +905,33 @@ mh_set_reply_regex (const char *str)
 	      err ? err : "");
 }
 
+const char *
+mh_charset (const char *dfl)
+{
+  const char *charset = mh_global_profile_get ("Charset", dfl);
+
+  if (!charset)
+    return NULL;
+  if (mu_c_strcasecmp (charset, "auto") == 0)
+    {
+      static char *saved_charset;
+
+      if (!saved_charset)
+	{
+	  /* Try to deduce the charset from LC_ALL variable */
+	  struct mu_lc_all lc_all;
+	  if (mu_parse_lc_all (getenv ("LC_ALL"), &lc_all, MU_LC_CSET) == 0)
+	    saved_charset = lc_all.charset; /* FIXME: Mild memory leak */
+	}
+      charset = saved_charset;
+    }
+  return charset;
+}
+
 int
 mh_decode_2047 (char *text, char **decoded_text)
 {
-  const char *charset = mh_global_profile_get ("Charset", NULL);
-
-  if (!charset)
-    return 1;
-  if (mu_c_strcasecmp (charset, "auto") == 0)
-    {
-      /* Try to deduce the charset from LC_ALL variable */
-
-      char *lc_all = getenv ("LC_ALL");
-      if (lc_all)
-	{
-	  char *sp;
-	  char *lang;
-	  char *terr;
-
-	  char *tmp = strdup (lc_all);
-	  lang = strtok_r (tmp, "_", &sp);
-	  terr = strtok_r (NULL, ".", &sp);
-	  charset = strtok_r (NULL, "@", &sp);
-
-	  if (!charset)
-	    charset = mu_charset_lookup (lang, terr);
-	  
-	  free (tmp);
-	}
-    }
-
+  const char *charset = mh_charset (NULL);
   if (!charset)
     return 1;
   
@@ -976,22 +1013,28 @@ mh_draft_message (const char *name, const char *msgspec, char **pname)
   mu_url_t url;
   size_t uid;
   int rc;
-  const char *urlstr;
   mu_mailbox_t mbox;
-
-  mbox = mh_open_folder (name, 0);
+  const char *path;
+  
+  mbox = mh_open_folder (name, MU_STREAM_RDWR);
   if (!mbox)
     return 1;
   
   mu_mailbox_get_url (mbox, &url);
-  urlstr = mu_url_to_string (url);
 
   if (strcmp (msgspec, "new") == 0)
     {
+      mu_property_t prop;
+      
       rc = mu_mailbox_uidnext (mbox, &uid);
       if (rc)
-	mu_error (_("cannot obtain sequence number for the new message: %s"),
-		  mu_strerror (rc));
+	{
+	  mu_error (_("cannot obtain sequence number for the new message: %s"),
+		    mu_strerror (rc));
+	  exit (1);
+	}
+      mu_mailbox_get_property (mbox, &prop);
+      mu_property_set_value (prop, "cur", mu_umaxtostr (0, uid), 1);
     }
   else
     {
@@ -1000,33 +1043,39 @@ mh_draft_message (const char *name, const char *msgspec, char **pname)
       
       argv[0] = (char*) msgspec;
       argv[1] = NULL;
-      rc = mh_msgset_parse (mbox, &msgset, 1, argv, "cur");
-      if (rc)
-	mu_error (_("invalid message number: %s"), msgspec);
-      else if (msgset.count > 1)
+      mh_msgset_parse (mbox, &msgset, 1, argv, "cur");
+      if (msgset.count > 1)
 	mu_error (_("only one message at a time!"));
       else
-	uid = msgset.list[0];
-
+	{
+	  mh_msgset_uids (mbox, &msgset);
+	  uid = msgset.list[0];
+	}
       mh_msgset_free (&msgset);
     }
-  
-  if (rc == 0)
+
+  mu_url_sget_path (url, &path);
+  rc = mu_asprintf (pname, "%s/%lu", path, (unsigned long) uid);
+  if (rc)
     {
-      const char *dir;
-      const char *msg;
-      size_t len;
-      
-      dir = urlstr + 3; /* FIXME */
-      
-      msg = mu_umaxtostr (0, uid);
-      len = strlen (dir) + 1 + strlen (msg) + 1;
-      *pname = xmalloc (len);
-      strcpy (*pname, dir);
-      strcat (*pname, "/");
-      strcat (*pname, msg);
+      mu_diag_funcall (MU_DIAG_ERROR, "mu_asprintf", NULL, rc);
+      exit (1);
     }
   mu_mailbox_close (mbox);
   mu_mailbox_destroy (&mbox);
   return rc;
 }
+
+char *
+mh_safe_make_file_name (const char *dir, const char *file)
+{
+  char *name = mu_make_file_name (dir, file);
+  if (!name)
+    {
+      mu_diag_funcall (MU_DIAG_ERROR, "mu_make_file_name", NULL, ENOMEM);
+      abort ();
+    }
+  return name;
+}
+
+			  
