@@ -1,9 +1,10 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2003, 2005 Free Software Foundation, Inc.
+   Copyright (C) 2003, 2005, 2006, 2007, 2009, 2010, 2011 Free Software
+   Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    GNU Mailutils is distributed in the hope that it will be useful,
@@ -12,9 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 /* MH mhl command */
 
@@ -100,7 +99,7 @@ compdecl (char **str, char **compname)
 {
   char *p;
   
-  for (p = *str; *p && !isspace (*p); p++)
+  for (p = *str; *p && !mu_isspace (*p); p++)
     {
       if (*p == ':')
 	{
@@ -132,13 +131,15 @@ parse_cleartext (locus_t *loc, mu_list_t formlist, char *str)
 static void
 parse_component (locus_t *loc, mu_list_t formlist, char *compname, char *str)
 {
+  int rc;
   mhl_stmt_t *stmt = stmt_alloc (stmt_component);
   stmt->v.component.name = compname;
-  if (mu_list_create (&stmt->v.component.format))
+  if ((rc = mu_list_create (&stmt->v.component.format)) != 0)
     {
-      mh_error (_("%s:%d: cannot create list"),
+      mu_error ("%s:%d: mu_list_create: %s",
 		loc->filename,
-		loc->line);
+		loc->line,
+		mu_strerror (rc));
       exit (1); /* FIXME */
     }
   parse_variable (loc, stmt->v.component.format, str);
@@ -148,47 +149,59 @@ parse_component (locus_t *loc, mu_list_t formlist, char *compname, char *str)
 static void
 parse_variable (locus_t *loc, mu_list_t formlist, char *str)
 {
-  int i;
-  int argc;
-  char **argv;
+  size_t i;
+  struct mu_wordsplit ws;
   mh_format_t fmt;
+  int wsflags;
   
-  if (mu_argcv_get (str, ",=", NULL, &argc, &argv))
+  if (strncmp (str, "ignores=", 8) == 0 && str[8] != '"')
     {
-      mh_error (_("%s:%d: cannot split string %s"),
+      /* A hack to allow for traditional use of "ignores=", i.e.
+	 as a single statement on a line, without double-quotes around
+	 the argument */
+      wsflags = MU_WRDSF_NOCMD | MU_WRDSF_NOVAR | MU_WRDSF_NOSPLIT;
+    }
+  else
+    {
+      ws.ws_delim = ",";
+      wsflags = MU_WRDSF_DEFFLAGS|MU_WRDSF_DELIM|
+	        MU_WRDSF_WS|MU_WRDSF_RETURN_DELIMS;
+    }
+  if (mu_wordsplit (str, &ws, wsflags))
+    {
+      mu_error ("%s:%d: mu_wordsplit(%s): %s",
 		loc->filename,
 		loc->line,
-		str);
+		str,
+		mu_wordsplit_strerror (&ws));
       exit (1);
     }
 
-  for (i = 0; i < argc; i++)
+  for (i = 0; i < ws.ws_wordc; i++)
     {
-      mhl_stmt_t *stmt = stmt_alloc (stmt_variable);
-      char *name = argv[i];
+      mhl_stmt_t *stmt;
+      char *name = ws.ws_wordv[i];
       char *value = NULL;
       mhl_variable_t *var;
       
+      value = strchr (name, '=');
+      if (value)
+	*value++ = 0;
+      stmt = stmt_alloc (stmt_variable);
       var = variable_lookup (name);
       if (!var)
 	{
-	  mh_error (_("%s:%d: unknown variable: %s"),
+	  mu_error (_("%s:%d: unknown variable: %s"),
 		    loc->filename,
 		    loc->line,
-		    argv[i]);
+		    name);
 	  exit (1);
 	}
 
-      if (i + 1 < argc && argv[i+1][0] == '=')
-	{
-	  i++;
-	  value = argv[++i];
-	}
-
       if ((var->type == dt_flag && value)
 	  || (var->type != dt_flag && !value))
 	{
-	  mh_error (_("%s:%d: wrong datatype for %s"),
+	  mu_error (_("%s:%d: wrong datatype for %s"),
 		    loc->filename,
 		    loc->line,
 		    var->name);
@@ -208,7 +221,7 @@ parse_variable (locus_t *loc, mu_list_t formlist, char *str)
 	case dt_format:
 	  if (mh_format_parse (value, &fmt))
 	    {
-	      mh_error (_("%s:%d: bad format string"),
+	      mu_error (_("%s:%d: bad format string"),
 			loc->filename,
 			loc->line);
 	      exit (1);
@@ -225,14 +238,13 @@ parse_variable (locus_t *loc, mu_list_t formlist, char *str)
       mu_list_append (formlist, stmt);
 
       i++;
-      if (i < argc && argv[i][0] != ',')
+      if (i < ws.ws_wordc && ws.ws_wordv[i][0] != ',')
 	{
-	  mh_error (_("%s:%d: syntax error"), loc->filename, loc->line);
+	  mu_error (_("%s:%d: syntax error"), loc->filename, loc->line);
 	  exit (1);
 	}
     }
-
-  mu_argcv_free (argc, argv);
+  mu_wordsplit_free (&ws);
 }
 
 static int
@@ -252,44 +264,39 @@ parse_line (locus_t *loc, mu_list_t formlist, char *str)
 mu_list_t 
 mhl_format_compile (char *name)
 {
-  FILE *fp;
+  mu_stream_t stream;
   mu_list_t formlist;
   char *buf = NULL;
-  size_t n = 0;
+  size_t size = 0, n;
   locus_t loc;
-  
-  fp = fopen (name, "r");
-  if (!fp)
+  int rc;
+
+  rc = mu_file_stream_create (&stream, name, MU_STREAM_READ);
+  if (rc)
     {
-      mh_error (_("Cannot open file %s: %s"), name, mu_strerror (errno));
+      mu_error (_("cannot open format file %s: %s"), name, mu_strerror (rc));
       return NULL;
     }
 
-  if (mu_list_create (&formlist))
+  if ((rc = mu_list_create (&formlist)) != 0)
     {
-      fclose (fp);
-      mh_error (_("Cannot create list"));
+      mu_diag_funcall (MU_DIAG_ERROR, "mu_list_create", NULL, rc);
+      mu_stream_destroy (&stream);
       return NULL;
     }
 
   loc.filename = name;
   loc.line = 1;
-  while (getline (&buf, &n, fp) > 0)
+  while (mu_stream_getline (stream, &buf, &size, &n) == 0 && n > 0)
     {
-      char *p = buf;
-
-      while (*p && isspace (*p))
-	;
-
-      if (*p == 0 || *p == ';')
-	continue;
-
-      parse_line (&loc, formlist, p);
+      char *p = mu_str_stripws (buf);
+      if (!(*p == 0 || *p == ';'))
+	parse_line (&loc, formlist, p);
       loc.line++;
     }
 
   free (buf);
-  fclose (fp);
+  mu_stream_destroy (&stream);
   
   return formlist;
 }
@@ -461,28 +468,36 @@ static void print (struct eval_env *env, char *str, int nloff);
 static int
 _comp_name (void *item, void *date)
 {
-  return strcasecmp (item, date) == 0;
+  return mu_c_strcasecmp (item, date) == 0;
 }
 
 int
-header_is_printed (struct eval_env *env, char *name)
+header_is_printed (struct eval_env *env, const char *name)
 {
-  return mu_list_do (env->printed_fields, _comp_name, name) == 1;
+  return mu_list_do (env->printed_fields, _comp_name, (void*) name) == 1;
 }
 
 int
-want_header (struct eval_env *env, char *name)
+want_header (struct eval_env *env, const char *name)
 {
-  char *p, *str = env->svar[S_IGNORES];
+  const char *p, *str;
 
-  for (p = strchrnul (str, ','); *str; p = strchrnul (str, ','))
+  for (str = env->svar[S_IGNORES], p = name; *str; str++)
     {
-      if (strncasecmp (name, str, p - str) == 0)
-	return 0;
-      str = p;
-      if (*str)
-	str++;
+      if (p)
+	{
+	  if (*p == 0 && *str == ',')
+	    break;
+	  if (mu_tolower (*p) == mu_tolower (*str))
+	    p++;
+	  else
+	    p = NULL;
+	}
+      else if (*str == ',')
+	p = name;
     }
+  if (p && *p == 0)
+    return 0;
   return 1;
 }
 
@@ -527,8 +542,8 @@ ovf_print (struct eval_env *env, char *str, int size, int nloff)
 	  if (env->svar[S_OVERFLOWTEXT])
 	    {
 	      int l = strlen (env->svar[S_OVERFLOWTEXT]);
-	      mu_stream_sequential_write (env->output,
-				       env->svar[S_OVERFLOWTEXT], l);
+	      mu_stream_write (env->output,
+			       env->svar[S_OVERFLOWTEXT], l, NULL);
 	      env->pos += l;
 	    }
 	}
@@ -538,12 +553,11 @@ ovf_print (struct eval_env *env, char *str, int size, int nloff)
 	    {
 	      goto_offset (env, env->ivar[I_OFFSET]);
 	      
-	      mu_stream_sequential_write (env->output, env->prefix,
-				       strlen (env->prefix));
+	      mu_stream_write (env->output, env->prefix,
+			       strlen (env->prefix), NULL);
 	      env->pos += strlen (env->prefix);
-	      
-	      goto_offset (env, nloff);
 	    }
+	  goto_offset (env, nloff);
 	}
       
       if (env->pos + len > env->ivar[I_WIDTH])
@@ -552,7 +566,7 @@ ovf_print (struct eval_env *env, char *str, int size, int nloff)
 	  len = env->ivar[I_WIDTH] - env->pos;
 	}
       
-      mu_stream_sequential_write (env->output, str, len);
+      mu_stream_write (env->output, str, len, NULL);
       env->pos += len;
       if (env->pos >= env->ivar[I_WIDTH])
 	newline (env);
@@ -571,16 +585,15 @@ print (struct eval_env *env, char *str, int nloff)
 	  newline (env);
 	  str++;
 	}
-      else
+      else if (*str)
 	{
-	  char *p = strchrnul (str, '\n');
-	  ovf_print (env, str, p - str, nloff);
-	  str = p;
-	  if (*p)
+	  size_t size = strcspn (str, "\n");
+	  ovf_print (env, str, size, nloff);
+	  str += size;
+	  if (*str == '\n')
 	    {
 	      newline (env);
-	      for (str++; *str && isspace (*str); str++)
-		;
+	      str = mu_str_skip_class (str + 1, MU_CTYPE_SPACE);
 	    }
 	}
     }
@@ -590,15 +603,15 @@ print (struct eval_env *env, char *str, int nloff)
 static void
 newline (struct eval_env *env)
 {
-  mu_stream_sequential_write (env->output, "\n", 1);
+  mu_stream_write (env->output, "\n", 1, NULL);
   env->pos = 0;
   if (env->ivar[I_LENGTH] && ++env->nlines >= env->ivar[I_LENGTH])
     {
       /* FIXME: Better to write it directly on the terminal */
       if (env->bvar[B_BELL])
-	mu_stream_sequential_write (env->output, "\a", 1);
+	mu_stream_write (env->output, "\a", 1, NULL);
       if (env->bvar[B_CLEARSCREEN])
-	mu_stream_sequential_write (env->output, "\f", 1);
+	mu_stream_write (env->output, "\f", 1, NULL);
       env->nlines = 0;
     }
 }
@@ -607,7 +620,7 @@ static void
 goto_offset (struct eval_env *env, int count)
 {
   for (; env->pos < count; env->pos++)
-    mu_stream_sequential_write (env->output, " ", 1);
+    mu_stream_write (env->output, " ", 1, NULL);
 }
 
 int
@@ -631,7 +644,7 @@ print_header_value (struct eval_env *env, char *val)
   if (env->bvar[B_UPPERCASE])
     {
       for (p = val; *p; p++)
-	*p = toupper (*p);
+	*p = mu_toupper (*p);
     }
 
   if (env->bvar[B_COMPRESS])
@@ -641,7 +654,7 @@ print_header_value (struct eval_env *env, char *val)
 
   if (env->bvar[B_LEFTADJUST])
     {
-      for (p = val; *p && isspace (*p); p++)
+      for (p = val; *p && mu_isspace (*p); p++)
 	;
     }
   else
@@ -651,19 +664,33 @@ print_header_value (struct eval_env *env, char *val)
   return 0;
 }
 
+void
+print_component_name (struct eval_env *env)
+{
+  if (!env->bvar[B_NOCOMPONENT])
+    {
+      print (env, env->svar[S_COMPONENT], 0);
+      if (mu_c_strcasecmp (env->svar[S_COMPONENT], "body"))
+	print (env, ": ", 0);
+    }
+}
+
 int
 eval_component (struct eval_env *env, char *name)
 {
   mu_header_t hdr;
   char *val;
-  
+
   mu_message_get_header (env->msg, &hdr);
   if (mu_header_aget_value (hdr, name, &val))
     return 0;
 
+  print_component_name (env);
   mu_list_append (env->printed_fields, name);
   print_header_value (env, val);
   free (val);
+  if (env->bvar[B_NEWLINE])
+    newline (env);
   return 0;
 }
 
@@ -675,14 +702,15 @@ eval_body (struct eval_env *env)
   char buf[128]; /* FIXME: Fixed size. Bad */
   size_t n;
   mu_body_t body = NULL;
-
+  int nl;
+  
   if (env->bvar[B_DISABLE_BODY])
     return 0;
-  
+
   env->prefix = env->svar[S_COMPONENT];
 
   mu_message_get_body (env->msg, &body);
-  mu_body_get_stream (body, &input);
+  mu_body_get_streamref (body, &input);
 
   if (env->bvar[B_DECODE])
     {
@@ -693,23 +721,25 @@ eval_body (struct eval_env *env)
       mu_header_aget_value (hdr, MU_HEADER_CONTENT_TRANSFER_ENCODING, &encoding);
       if (encoding)
 	{
-	  int rc = mu_filter_create(&dstr, input, encoding,
-				 MU_FILTER_DECODE, MU_STREAM_READ);
+	  int rc = mu_filter_create (&dstr, input, encoding,
+				     MU_FILTER_DECODE, 
+				     MU_STREAM_READ);
 	  if (rc == 0)
 	    input = dstr;
 	  free (encoding);
 	}
     }
   
-  mu_stream_seek (input, 0, SEEK_SET);
-  while (mu_stream_sequential_readline (input, buf, sizeof buf, &n) == 0
+  while (mu_stream_readline (input, buf, sizeof buf, &n) == 0
 	 && n > 0)
     {
-      buf[n] = 0;
+      goto_offset (env, env->ivar[I_OFFSET]);
       print (env, buf, 0);
+      nl = buf[n-1] == '\n';
     }
-  if (dstr)
-    mu_stream_destroy (&dstr, mu_stream_get_owner (dstr));
+  mu_stream_destroy (&input);
+  if (!nl && env->bvar[B_NEWLINE])
+    newline (env);
   return 0;
 }
 
@@ -718,25 +748,31 @@ eval_extras (struct eval_env *env)
 {
   mu_header_t hdr;
   size_t i, num;
-  char buf[512];
+  char *str;
 
+  print_component_name (env);
   mu_message_get_header (env->msg, &hdr);
   mu_header_get_field_count (hdr, &num);
   for (i = 1; i <= num; i++)
     {
-      mu_header_get_field_name (hdr, i, buf, sizeof buf, NULL);
-      if (want_header (env, buf)
-	  && !header_is_printed (env, buf))
+      if (mu_header_aget_field_name (hdr, i, &str))
+	break;
+      if (want_header (env, str)
+	  && !header_is_printed (env, str))
 	{
 	  goto_offset (env, env->ivar[I_OFFSET]);
-	  print (env, buf, 0);
-	  print (env, ":", 0);
-	  mu_header_get_field_value (hdr, i, buf, sizeof buf, NULL);
-	  print_header_value (env, buf);
-	  if (env->bvar[B_NEWLINE])
+	  print (env, str, 0);
+	  print (env, ": ", 0);
+	  free (str);
+	  mu_header_aget_field_value (hdr, i, &str);
+	  print_header_value (env, str);
+	  if (i < num && env->bvar[B_NEWLINE])
 	    newline (env);
 	}
+      free (str);
     }
+  if (env->bvar[B_NEWLINE])
+    newline (env);
   return 0;
 }
 
@@ -752,13 +788,6 @@ eval_comp (struct eval_env *env, char *compname, mu_list_t format)
   if (!lenv.svar[S_COMPONENT])
     lenv.svar[S_COMPONENT] = compname;
   
-  if (!lenv.bvar[B_NOCOMPONENT])
-    {
-      print (&lenv, lenv.svar[S_COMPONENT], 0);
-      if (strcmp (compname, "body"))
-	print (&lenv, ":", 0);
-    }
-
   if (strcmp (compname, "extras") == 0)
     eval_extras (&lenv);
   else if (strcmp (compname, "body") == 0)
@@ -766,9 +795,6 @@ eval_comp (struct eval_env *env, char *compname, mu_list_t format)
   else
     eval_component (&lenv, compname);
   
-  if (lenv.bvar[B_NEWLINE])
-    newline (&lenv);
-
   env->pos = lenv.pos;
   env->nlines = lenv.nlines;
   
