@@ -1,5 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2005, 2007 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2005, 2007, 2009, 2010, 2011 Free
+   Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,16 +13,14 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include "pop3d.h"
 
 /* DELE adds a message number to the list of messages to be deleted on QUIT */
 
 int
-pop3d_dele (const char *arg)
+pop3d_dele (char *arg)
 {
   size_t num;
   mu_message_t msg;
@@ -40,6 +39,6 @@ pop3d_dele (const char *arg)
 
   mu_message_get_attribute (msg, &attr);
   pop3d_mark_deleted (attr);
-  pop3d_outf ("+OK Message %s marked\r\n", mu_umaxtostr (0, num));
+  pop3d_outf ("+OK Message %s marked\n", mu_umaxtostr (0, num));
   return OK;
 }
