@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 
-   2005, 2007, 2008, 2009 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2007, 2008,
+   2009, 2010, 2011 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,13 +13,12 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include "pop3d.h"
 #include "mailutils/pam.h"
 #include "mailutils/libargp.h"
+#include "mailutils/pop3.h"
 #include "tcpwrap.h"
 
 mu_mailbox_t mbox;
@@ -32,6 +31,10 @@ unsigned int idle_timeout;
 int pop3d_transcript;
 int debug_mode;
 int tls_required;
+int pop3d_xlines;
+char *apop_database_name = APOP_PASSFILE;
+int apop_database_safety;
+int apop_database_safety_set;
 
 #ifdef WITH_TLS
 int tls_available;
@@ -49,21 +52,14 @@ char *login_stat_file = LOGIN_STAT_FILE;
 #endif
 
 unsigned expire = EXPIRE_NEVER; /* Expire messages after this number of days */
-int expire_on_exit = 0;       /* Delete expired messages on exit */
+int expire_on_exit = 0;         /* Delete expired messages on exit */
 
 static error_t pop3d_parse_opt  (int key, char *arg, struct argp_state *astate);
 
 const char *program_version = "pop3d (" PACKAGE_STRING ")";
-static char doc[] = N_("GNU pop3d -- the POP3 daemon");
+static char doc[] = N_("GNU pop3d -- the POP3 daemon.");
 
-#define OPT_LOGIN_DELAY     257
-#define OPT_STAT_FILE       258
-#define OPT_EXPIRE          259
-#define OPT_EXPIRE_ON_EXIT  260
-#define OPT_TLS_REQUIRED    261
-#define OPT_BULLETIN_SOURCE 262
-#define OPT_BULLETIN_DB     263
-#define OPT_FOREGROUND      264
+#define OPT_FOREGROUND      256
 
 static struct argp_option options[] = {
 #define GRP 0
@@ -73,61 +69,58 @@ static struct argp_option options[] = {
     N_("runs in daemon mode with a maximum of NUMBER children"), GRP+1 },
 #undef GRP
 
-#define GRP 5
-  {"undelete", 'u', NULL, OPTION_HIDDEN,
-   N_("undelete all messages on startup"), GRP+1},
-  {"expire", OPT_EXPIRE, N_("DAYS"), OPTION_HIDDEN,
-   N_("expire read messages after the given number of days"), GRP+1},
-  {"delete-expired", OPT_EXPIRE_ON_EXIT, NULL, OPTION_HIDDEN,
-   N_("delete expired messages upon closing the mailbox"), GRP+1},
-#ifdef WITH_TLS
-  {"tls-required", OPT_TLS_REQUIRED, NULL, OPTION_HIDDEN,
-   N_("always require STLS before entering authentication phase")},
-#endif
-#undef GRP
-  
-#define GRP 10
-#ifdef ENABLE_LOGIN_DELAY
-  {"login-delay", OPT_LOGIN_DELAY, N_("SECONDS"), OPTION_HIDDEN,
-   N_("allowed delay between two successive logins"), GRP+1},
-  {"stat-file", OPT_STAT_FILE, N_("FILENAME"), OPTION_HIDDEN,
-   N_("name of login statistics file"), GRP+1},
-#endif
-  
-#undef GRP
-
-#define GRP 20
-  { "bulletin-source", OPT_BULLETIN_SOURCE, N_("MBOX"), OPTION_HIDDEN,
-    N_("set source mailbox to get bulletins from"), GRP+1 },
-#ifdef USE_DBM
-  { "bulletin-db", OPT_BULLETIN_DB, N_("FILE"), OPTION_HIDDEN,
-    N_("set the bulletin database file name"), GRP+1 },
-#endif
-#undef GRP
-  
   {NULL, 0, NULL, 0, NULL, 0}
 };
 
 static int
-cb_bulletin_source (mu_debug_t debug, void *data, mu_config_value_t *val)
+cb_bulletin_source (void *data, mu_config_value_t *val)
 {
-  if (mu_cfg_assert_value_type (val, MU_CFG_STRING, debug))
+  if (mu_cfg_assert_value_type (val, MU_CFG_STRING))
     return 1;
   set_bulletin_source (val->v.string); /* FIXME: Error reporting? */
   return 0;
 }
 
-#ifdef USE_DBM
 static int
-cb_bulletin_db (mu_debug_t debug, void *data, mu_config_value_t *val)
+cb2_file_safety_checks (const char *name, void *data)
 {
-  if (mu_cfg_assert_value_type (val, MU_CFG_STRING, debug))
+  if (mu_file_safety_compose (data, name, MU_FILE_SAFETY_ALL))
+    mu_error (_("unknown keyword: %s"), name);
+  return 0;
+}
+
+static int
+cb_apop_safety_checks (void *data, mu_config_value_t *arg)
+{
+  apop_database_safety_set = 1;
+  return mu_cfg_string_value_cb (arg, cb2_file_safety_checks,
+				 &apop_database_safety);
+}
+
+#ifdef ENABLE_DBM
+static int
+cb_bulletin_db (void *data, mu_config_value_t *val)
+{
+  if (mu_cfg_assert_value_type (val, MU_CFG_STRING))
     return 1;
   set_bulletin_db (val->v.string); /* FIXME: Error reporting? */
   return 0;
 }
 #endif
 
+struct pop3d_srv_config
+{
+  struct mu_srv_config m_cfg;
+  int tls;
+};
+
+static struct mu_cfg_param pop3d_srv_param[] = {
+  { "tls", mu_cfg_bool, NULL, mu_offsetof (struct pop3d_srv_config, tls), NULL,
+    N_("Use TLS encryption for this server")
+  },
+  { NULL }
+};
+    
 static struct mu_cfg_param pop3d_cfg_param[] = {
   { "undelete", mu_cfg_bool, &undelete_on_startup, 0, NULL,
     N_("On startup, clear deletion marks from all the messages.") },
@@ -136,6 +129,26 @@ static struct mu_cfg_param pop3d_cfg_param[] = {
     N_("days") },
   { "delete-expired", mu_cfg_bool, &expire_on_exit, 0, NULL,
     N_("Delete expired messages upon closing the mailbox.") },
+  { "scan-lines", mu_cfg_bool, &pop3d_xlines, 0, NULL,
+    N_("Output the number of lines in the message in its scan listing.") },
+  { "apop-database-file", mu_cfg_string, &apop_database_name, 0, NULL,
+    N_("set APOP database file name or URL") },
+  { "apop-database-safety", mu_cfg_callback, NULL, 0, cb_apop_safety_checks,
+    N_("Configure safety checks for APOP database files.  Argument is a list or "
+       "sequence of check names optionally prefixed with '+' to enable or "
+       "'-' to disable the corresponding check.  Valid check names are:\n"
+       "\n"
+       "  none          disable all checks\n"
+       "  all           enable all checks\n"
+       "  gwrfil        forbid group writable files\n"
+       "  awrfil        forbid world writable files\n"
+       "  grdfil        forbid group readable files\n"
+       "  ardfil        forbid world writable files\n"
+       "  linkwrdir     forbid symbolic links in group or world writable directories\n"
+       "  gwrdir        forbid files in group writable directories\n"
+       "  awrdir        forbid files in world writable directories\n"),
+    N_("arg: list") },  
+    
 #ifdef WITH_TLS
   { "tls-required", mu_cfg_bool, &tls_required, 0, NULL,
      N_("Always require STLS before entering authentication phase.") },
@@ -149,13 +162,18 @@ static struct mu_cfg_param pop3d_cfg_param[] = {
   { "bulletin-source", mu_cfg_callback, NULL, 0, cb_bulletin_source,
     N_("Get bulletins from the specified mailbox."),
     N_("url") },
-#ifdef USE_DBM
+#ifdef ENABLE_DBM
   { "bulletin-db", mu_cfg_callback, NULL, 0, cb_bulletin_db,
     N_("Set the bulletin database file name."),
     N_("file") },
 #endif
+  { "output-buffer-size", mu_cfg_size, &pop3d_output_bufsize, 0, NULL,
+    N_("Size of the output buffer.") },
+  { "mandatory-locking", mu_cfg_section },
   { ".server", mu_cfg_section, NULL, 0, NULL,
     N_("Server configuration.") },
+  { "transcript", mu_cfg_bool, &pop3d_transcript, 0, NULL,
+    N_("Set global transcript mode.") },
   TCP_WRAPPERS_CONFIG
   { NULL }
 };
@@ -176,75 +194,36 @@ static const char *pop3d_argp_capa[] = {
   "mailbox",
   "locking",
   "logging",
-  "license",
   NULL
 };
 
 static error_t
 pop3d_parse_opt (int key, char *arg, struct argp_state *astate)
 {
-  static struct mu_argp_node_list lst;
+  static mu_list_t lst;
   
   switch (key)
     {
     case 'd':
-      mu_argp_node_list_new (&lst, "mode", "daemon");
+      mu_argp_node_list_new (lst, "mode", "daemon");
       if (arg)
-	mu_argp_node_list_new (&lst, "max-children", arg);
+	mu_argp_node_list_new (lst, "max-children", arg);
       break;
 
     case 'i':
-      mu_argp_node_list_new (&lst, "mode", "inetd");
+      mu_argp_node_list_new (lst, "mode", "inetd");
       break;
 
     case OPT_FOREGROUND:
-      mu_argp_node_list_new (&lst, "foreground", "yes");
+      mu_argp_node_list_new (lst, "foreground", "yes");
       break;
       
-    case 'u':
-      mu_argp_node_list_new (&lst, "undelete", "yes");
-      break;
-
-#ifdef ENABLE_LOGIN_DELAY
-    case OPT_LOGIN_DELAY:
-      mu_argp_node_list_new (&lst, "login-delay", arg);
-      break;
-
-    case OPT_STAT_FILE:
-      mu_argp_node_list_new (&lst, "stat-file", arg);
-      break;
-#endif  
- 
-    case OPT_EXPIRE:
-      mu_argp_node_list_new (&lst, "expire", arg);
-      break;
-
-    case OPT_EXPIRE_ON_EXIT:
-      mu_argp_node_list_new (&lst, "delete-expired", "yes");
-      break;
-
-#ifdef WITH_TLS
-    case OPT_TLS_REQUIRED:
-      mu_argp_node_list_new (&lst, "tls-required", "yes");
-      break;
-#endif
-
-    case OPT_BULLETIN_SOURCE:
-      mu_argp_node_list_new (&lst, "bulletin-source", arg);
-      break;
-      
-#ifdef USE_DBM
-    case OPT_BULLETIN_DB:
-      mu_argp_node_list_new (&lst, "bulletin-db", arg);
-      break;
-#endif
-      
     case ARGP_KEY_INIT:
       mu_argp_node_list_init (&lst);
       break;
       
     case ARGP_KEY_FINI:
-      mu_argp_node_list_finish (&lst, NULL, NULL);
+      mu_argp_node_list_finish (lst, NULL, NULL);
       break;
       
     default:
@@ -266,7 +245,7 @@ pop3d_get_client_address (int fd, struct sockaddr_in *pcs)
     }
   else
     {
-      int len = sizeof *pcs;
+      socklen_t len = sizeof *pcs;
       if (getpeername (fd, (struct sockaddr*) pcs, &len) < 0)
 	{
 	  mu_diag_output (MU_DIAG_ERROR,
@@ -281,11 +260,11 @@ pop3d_get_client_address (int fd, struct sockaddr_in *pcs)
 /* The main part of the daemon. This function reads input from the client and
    executes the proper functions. Also handles the bulk of error reporting.
    Arguments:
-      fd        --  socket descriptor (for diagnostics)
-      infile    --  input stream
-      outfile   --  output stream */
+      ifd       --  input descriptor
+      ofd       --  output descriptor
+      tls       --  initiate encrypted connection */
 int
-pop3d_mainloop (int fd, FILE *infile, FILE *outfile)
+pop3d_mainloop (int ifd, int ofd, int tls)
 {
   int status = OK;
   char buffer[512];
@@ -294,26 +273,20 @@ pop3d_mainloop (int fd, FILE *infile, FILE *outfile)
 
   mu_set_signals (pop3d_child_signal, sigtab, MU_ARRAY_SIZE (sigtab));
 
-  pop3d_setio (infile, outfile);
+  pop3d_setio (ifd, ofd, tls);
 
-  state = initial_state;
+  state = tls ? AUTHORIZATION : initial_state;
 
   /* Prepare the shared secret for APOP.  */
   {
     char *local_hostname;
-    local_hostname = mu_alloc (MAXHOSTNAMELEN + 1);
-
-    /* Get our canonical hostname. */
-    {
-      struct hostent *htbuf;
-      gethostname (local_hostname, MAXHOSTNAMELEN);
-      htbuf = gethostbyname (local_hostname);
-      if (htbuf)
-	{
-	  free (local_hostname);
-	  local_hostname = strdup (htbuf->h_name);
-	}
-    }
+    
+    status = mu_get_host_name (&local_hostname);
+    if (status)
+      {
+        mu_diag_funcall (MU_DIAG_ERROR, "mu_get_host_name", NULL, status);
+        exit (EXIT_FAILURE);
+      }
 
     md5shared = mu_alloc (strlen (local_hostname) + 51);
 
@@ -323,7 +296,7 @@ pop3d_mainloop (int fd, FILE *infile, FILE *outfile)
   }
 
   /* Lets boogie.  */
-  pop3d_outf ("+OK POP3 Ready %s\r\n", md5shared);
+  pop3d_outf ("+OK POP3 Ready %s\n", md5shared);
 
   while (state != UPDATE && state != ABORT)
     {
@@ -336,11 +309,6 @@ pop3d_mainloop (int fd, FILE *infile, FILE *outfile)
       buf = pop3d_readline (buffer, sizeof (buffer));
       pop3d_parse_command (buf, &cmd, &arg);
 
-      /* The mailbox size needs to be check to make sure that we are in
-	 sync.  Some other applications may not respect the *.lock or
-	 the lock may be stale because downloading on slow modem.
-	 We rely on the size of the mailbox for the check and bail if out
-	 of sync.  */
       if (state == TRANSACTION && !mu_mailbox_is_updated (mbox))
 	{
 	  static mu_off_t mailbox_size;
@@ -354,19 +322,15 @@ pop3d_mainloop (int fd, FILE *infile, FILE *outfile)
 	}
 
       /* Refresh the Lock.  */
-      pop3d_touchlock ();
+      manlock_touchlock (mbox);
 
-      if (strlen (arg) > POP_MAXCMDLEN || strlen (cmd) > POP_MAXCMDLEN)
-	status = ERR_TOO_LONG;
-      else if (strlen (cmd) > 4)
-	status = ERR_BAD_CMD;
-      else if ((handler = pop3d_find_command (cmd)) != NULL)
+      if ((handler = pop3d_find_command (cmd)) != NULL)
 	status = handler (arg);
       else
 	status = ERR_BAD_CMD;
 
       if (status != OK)
-	pop3d_outf ("-ERR %s\r\n", pop3d_error_string (status));
+	pop3d_outf ("-ERR %s\n", pop3d_error_string (status));
     }
 
   pop3d_bye ();
@@ -375,12 +339,16 @@ pop3d_mainloop (int fd, FILE *infile, FILE *outfile)
 }
 
 int
-pop3d_connection (int fd, struct sockaddr *sa, int salen, void *data,
-		  mu_ip_server_t srv, time_t timeout, int transcript)
+pop3d_connection (int fd, struct sockaddr *sa, int salen,
+		  struct mu_srv_config *pconf,
+		  void *data)
 {
-  idle_timeout = timeout;
-  pop3d_transcript = transcript;
-  pop3d_mainloop (fd, fdopen (fd, "r"), fdopen (fd, "w"));
+  struct pop3d_srv_config *cfg = (struct pop3d_srv_config *) pconf;
+  
+  idle_timeout = pconf->timeout;
+  pop3d_transcript = pconf->transcript;
+
+  pop3d_mainloop (fd, fd, cfg->tls);
   return 0;
 }
 
@@ -390,13 +358,22 @@ pop3d_alloc_die ()
   pop3d_abquit (ERR_NO_MEM);
 }
 
+#ifdef ENABLE_DBM
+static void
+set_dbm_safety ()
+{
+  mu_url_t hints = mu_dbm_get_hint ();
+  const char *param[] = { "+all" };
+  mu_url_add_param (hints, 1, param);
+}
+#endif
+
 int
 main (int argc, char **argv)
 {
   struct group *gr;
   int status = OK;
-  static int sigtab[] = { SIGILL, SIGBUS, SIGFPE, SIGSEGV, SIGSTOP, SIGPIPE,
-			  SIGABRT };
+  static int sigtab[] = { SIGILL, SIGBUS, SIGFPE, SIGSEGV, SIGSTOP, SIGPIPE };
 
   /* Native Language Support */
   MU_APP_INIT_NLS ();
@@ -409,12 +386,14 @@ main (int argc, char **argv)
   mu_gocs_register ("tls", mu_tls_module_init);
 #endif /* WITH_TLS */
   mu_tcpwrapper_cfg_init ();
+  manlock_cfg_init ();
   mu_acl_cfg_init ();
-  mu_m_server_cfg_init ();
+  mu_m_server_cfg_init (pop3d_srv_param);
   
-  mu_argp_init (program_version, NULL);
+  mu_argp_init (NULL, NULL);
   	
   mu_m_server_create (&server, program_version);
+  mu_m_server_set_config_size (server, sizeof (struct pop3d_srv_config));
   mu_m_server_set_conn (server, pop3d_connection);
   mu_m_server_set_prefork (server, mu_tcp_wrapper_prefork);
   mu_m_server_set_mode (server, MODE_INTERACTIVE);
@@ -425,6 +404,13 @@ main (int argc, char **argv)
   mu_m_server_set_strexit (server, mu_strexit);
 
   mu_alloc_die_hook = pop3d_alloc_die;
+
+  mu_log_syslog = 1;
+  manlock_mandatory_locking = 1;
+
+#ifdef ENABLE_DBM
+  set_dbm_safety ();
+#endif
   
   if (mu_app_init (&argp, pop3d_argp_capa, pop3d_cfg_param, 
 		   argc, argv, 0, NULL, server))
@@ -474,18 +460,8 @@ main (int argc, char **argv)
   /* Set the signal handlers.  */
   mu_set_signals (pop3d_master_signal, sigtab, MU_ARRAY_SIZE (sigtab));
 
-  /* Set up for syslog.  */
-  openlog (MU_LOG_TAG (), LOG_PID, mu_log_facility);
-  /* Redirect any stdout error from the library to syslog, they
-     should not go to the client.  */
-  {
-    mu_debug_t debug;
-
-    mu_diag_get_debug (&debug);
-    mu_debug_set_print (debug, mu_diag_syslog_printer, NULL);
-    
-    mu_debug_default_printer = mu_debug_syslog_printer;
-  }
+  mu_stdstream_strerr_setup (mu_log_syslog ?
+			     MU_STRERR_SYSLOG : MU_STRERR_STDERR);
   
   umask (S_IROTH | S_IWOTH | S_IXOTH);	/* 007 */
 
@@ -494,7 +470,7 @@ main (int argc, char **argv)
   tls_available = mu_check_tls_environment ();
   if (tls_available)
     {
-      tls_available = mu_init_tls_libs ();
+      tls_available = mu_init_tls_libs (1);
       if (tls_available)
 	enable_stls ();
     }
@@ -512,7 +488,7 @@ main (int argc, char **argv)
     {
       /* Make sure we are in the root directory.  */
       chdir ("/");
-      status = pop3d_mainloop (fileno (stdin), stdin, stdout);
+      status = pop3d_mainloop (MU_STDIN_FD, MU_STDOUT_FD, 0);
     }
   
   if (status)
