@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002, 2005,
-   2007 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2002, 2005, 2007, 2009, 2010, 2011
+   Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,9 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include "pop3d.h"
 
@@ -28,7 +26,7 @@
 static void pop3d_fix_mark ();
 
 int
-pop3d_quit (const char *arg)
+pop3d_quit (char *arg)
 {
   int err = OK;
   if (strlen (arg) != 0)
@@ -36,18 +34,18 @@ pop3d_quit (const char *arg)
 
   if (state == TRANSACTION)
     {
-      pop3d_unlock ();
       pop3d_fix_mark ();
 
       if (mu_mailbox_flush (mbox, 1) != 0)
 	err = ERR_FILE;
       if (mu_mailbox_close (mbox) != 0) 
 	err = ERR_FILE;
+      manlock_unlock (mbox);
       mu_mailbox_destroy (&mbox);
-      mu_diag_output (MU_DIAG_INFO, _("Session ended for user: %s"), username);
+      mu_diag_output (MU_DIAG_INFO, _("session ended for user: %s"), username);
     }
   else
-    mu_diag_output (MU_DIAG_INFO, _("Session ended for no user"));
+    mu_diag_output (MU_DIAG_INFO, _("session ended for no user"));
 
   state = UPDATE;
   update_login_delay (username);
@@ -55,7 +53,7 @@ pop3d_quit (const char *arg)
   free (md5shared);
 
   if (err == OK)
-    pop3d_outf ("+OK\r\n");
+    pop3d_outf ("+OK\n");
   return err;
 }
 
