@@ -1,5 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2003, 2007 Free Software Foundation, Inc.
+   Copyright (C) 2003, 2007, 2009, 2010, 2011 Free Software Foundation,
+   Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,9 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include "pop3d.h"
 
@@ -23,7 +22,7 @@
 #ifdef WITH_TLS
 
 int
-pop3d_stls (const char *arg)
+pop3d_stls (char *arg)
 {
   if (strlen (arg) != 0)
     return ERR_BAD_ARGS;
@@ -34,13 +33,17 @@ pop3d_stls (const char *arg)
   if (tls_done)
     return ERR_TLS_ACTIVE;
 
-  pop3d_outf ("+OK Begin TLS negotiation\r\n");
+  pop3d_outf ("+OK Begin TLS negotiation\n");
   pop3d_flush_output ();
 
-  tls_done = pop3d_init_tls_server ();
+  tls_done = pop3d_init_tls_server () == 0;
 
   if (!tls_done)
-    return ERR_UNKNOWN;
+    {
+      mu_diag_output (MU_DIAG_ERROR, _("Session terminated"));
+      state = ABORT;
+      return ERR_UNKNOWN;
+    }
 
   state = AUTHORIZATION;  /* Confirm we're in this state. Necessary for
 			     --tls-required to work */
