@@ -1,5 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2005, 2007 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2005, 2007, 2010, 2011 Free Software
+   Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -22,17 +23,14 @@ int
 pop3d_top (char *arg)
 {
   size_t mesgno;
-  int lines;
+  unsigned long lines;
   mu_message_t msg;
   mu_attribute_t attr;
   mu_header_t hdr;
   mu_body_t body;
-  mu_stream_t stream;
-  char *mesgc, *linesc;
-  char buf[BUFFERSIZE];
-  size_t n;
-  off_t off;
-
+  mu_stream_t hstream, bstream;
+  char *mesgc, *linesc, *p;
+  
   if (strlen (arg) == 0)
     return ERR_BAD_ARGS;
 
@@ -40,11 +38,14 @@ pop3d_top (char *arg)
     return ERR_WRONG_STATE;
 
   pop3d_parse_command (arg, &mesgc, &linesc);
+  if (linesc[0] == 0)
+    return ERR_BAD_ARGS;
   
-  mesgno = strtoul (mesgc, NULL, 10);
-  lines = *linesc ? strtol (linesc, NULL, 10) : -1;
-
-  if (lines < 0)
+  mesgno = strtoul (mesgc, &p, 10);
+  if (*p)
+    return ERR_BAD_ARGS;
+  lines = strtoul (linesc, &p, 10);
+  if (*p)
     return ERR_BAD_ARGS;
 
   if (mu_mailbox_get_message (mbox, mesgno, &msg) != 0)
@@ -55,59 +56,23 @@ pop3d_top (char *arg)
     return ERR_MESG_DELE;
   pop3d_mark_retr (attr);
   
-  pop3d_outf ("+OK\r\n");
-
   /* Header.  */
   mu_message_get_header (msg, &hdr);
-  mu_header_get_stream (hdr, &stream);
-  off = n = 0;
-  while (mu_stream_readline (stream, buf, sizeof(buf), off, &n) == 0
-	 && n > 0)
+  if (mu_header_get_streamref (hdr, &hstream))
+    return ERR_UNKNOWN;
+  mu_message_get_body (msg, &body);
+  if (mu_body_get_streamref (body, &bstream))
     {
-      /* Nuke the trainline newline.  */
-      if (buf[n - 1] == '\n')
-	{
-	  buf[n - 1] = '\0';
-	  pop3d_outf ("%s\r\n", buf);
-	}
-      else
-	pop3d_outf ("%s", buf);
-      off += n;
+      mu_stream_unref (hstream);
+      return ERR_UNKNOWN;
     }
+    
+  pop3d_outf ("+OK\n");
 
-  /* Lines of body.  */
-  if (lines)
-    {
-      int prev_nl = 1;
+  pop3d_send_payload (hstream, bstream, lines);
 
-      mu_message_get_body (msg, &body);
-      mu_body_get_stream (body, &stream);
-      n = off = 0;
-      while (mu_stream_readline (stream, buf, sizeof(buf), off, &n) == 0
-	     && n > 0 && lines > 0)
-	{
-	  if (prev_nl && buf[0] == '.')
-	    pop3d_outf (".");
-      
-	  if (buf[n - 1] == '\n')
-	    {
-	      buf[n - 1] = '\0';
-	      pop3d_outf ("%s\r\n", buf);
-	      prev_nl = 1;
-	      lines--;
-	    }
-	  else
-	    {
-	      pop3d_outf ("%s", buf);
-	      prev_nl = 0;
-	    }
-	  off += n;
-	}
-      if (!prev_nl)
-	pop3d_outf ("\r\n");
-    }
-
-  pop3d_outf (".\r\n");
+  mu_stream_unref (hstream);
+  mu_stream_unref (bstream);
 
   return OK;
 }
