@@ -1,6 +1,5 @@
-/*
-   GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2009 Free Software Foundation, Inc.
+/* GNU Mailutils -- a suite of utilities for electronic mail
+   Copyright (C) 2009, 2010, 2011 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -13,10 +12,8 @@
    Lesser General Public License for more details.
 
    You should have received a copy of the GNU Lesser General
-   Public License along with this library; if not, write to the
-   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301 USA
-*/
+   Public License along with this library.  If not, see 
+   <http://www.gnu.org/licenses/>. */
 
 #include "libmu_py.h"
 
@@ -342,22 +339,6 @@ api_mailbox_get_size (PyObject *self, PyObject *args)
   return status_object (status, PyInt_FromLong (size));
 }
 
-static PyObject *
-api_mailbox_get_debug (PyObject *self, PyObject *args)
-{
-  int status;
-  PyMailbox *py_mbox;
-  PyDebug *py_dbg = PyDebug_NEW ();
-
-  if (!PyArg_ParseTuple (args, "O!", &PyMailboxType, &py_mbox))
-    return NULL;
-
-  status = mu_mailbox_get_debug (py_mbox->mbox, &py_dbg->dbg);
-
-  Py_INCREF (py_dbg);
-  return status_object (status, (PyObject *)py_dbg);
-}
-
 static PyObject *
 api_mailbox_get_folder (PyObject *self, PyObject *args)
 {
@@ -447,9 +428,6 @@ static PyMethodDef methods[] = {
   { "get_size", (PyCFunction) api_mailbox_get_size, METH_VARARGS,
     "" },
 
-  { "get_debug", (PyCFunction) api_mailbox_get_debug, METH_VARARGS,
-    "" },
-
   { "get_folder", (PyCFunction) api_mailbox_get_folder, METH_VARARGS,
     "" },
 
